@@ -5,9 +5,16 @@
 #include "core/libdivecomputer.h"
 #include <QTimer>
 #include <QDebug>
+#include <QLoggingCategory>
+#include <QRegularExpression>
+#include <QElapsedTimer>
+#include <QCoreApplication>
 
 extern QMap<QString, dc_descriptor_t *> descriptorLookup;
 
+namespace {
+	QHash<QString, QBluetoothDeviceInfo> btDeviceInfo;
+}
 BTDiscovery *BTDiscovery::m_instance = NULL;
 
 static dc_descriptor_t *getDeviceType(QString btName)
@@ -17,8 +24,7 @@ static dc_descriptor_t *getDeviceType(QString btName)
 
 	if (btName.startsWith("OSTC")) {
 		vendor = "Heinrichs Weikamp";
-		if (btName.mid(4,2) == "3#") product = "OSTC 3";
-		else if (btName.mid(4,2) == "3+") product = "OSTC 3+";
+		if (btName.mid(4,1) == "3") product = "OSTC Plus";
 		else if (btName.mid(4,2) == "s#") product = "OSTC Sport";
 		else if (btName.mid(4,2) == "s ") product = "OSTC Sport";
 		else if (btName.mid(4,2) == "4-") product = "OSTC 4";
@@ -30,8 +36,8 @@ static dc_descriptor_t *getDeviceType(QString btName)
 	}
 
 	if (btName.startsWith("Predator") ||
-            btName.startsWith("Petrel") ||
-            btName.startsWith("Perdix") ||
+	    btName.startsWith("Petrel") ||
+	    btName.startsWith("Perdix") ||
 	    btName.startsWith("Teric")) {
 		vendor = "Shearwater";
 		if (btName.startsWith("Petrel")) product = "Petrel"; // or petrel 2?
@@ -63,6 +69,11 @@ static dc_descriptor_t *getDeviceType(QString btName)
 		product = "Quad";
 	}
 
+	if (btName.contains(QRegularExpression("^FQ\\d{6}$"))) {
+		vendor = "Aqualung";
+		product = "i770R";
+	}
+
 	if (!vendor.isEmpty() && !product.isEmpty())
 		return descriptorLookup.value(vendor + product);
 
@@ -70,7 +81,7 @@ static dc_descriptor_t *getDeviceType(QString btName)
 }
 
 BTDiscovery::BTDiscovery(QObject*) : m_btValid(false),
-	discoveryAgent(NULL)
+	discoveryAgent(nullptr)
 {
 	if (m_instance) {
 		qDebug() << "trying to create an additional BTDiscovery object";
@@ -78,6 +89,7 @@ BTDiscovery::BTDiscovery(QObject*) : m_btValid(false),
 	}
 	m_instance = this;
 #if defined(BT_SUPPORT)
+	QLoggingCategory::setFilterRules(QStringLiteral("qt.bluetooth* = true"));
 	BTDiscoveryReDiscover();
 #endif
 }
@@ -96,25 +108,24 @@ void BTDiscovery::BTDiscoveryReDiscover()
 	if (1) {
 #endif
 		m_btValid = true;
-#if defined(Q_OS_IOS) || (defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID))
-		discoveryAgent = new QBluetoothDeviceDiscoveryAgent(this);
-		connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::deviceDiscovered, this, &BTDiscovery::btDeviceDiscovered);
+#if !defined(Q_OS_ANDROID)
+		if (discoveryAgent == nullptr) {
+			discoveryAgent = new QBluetoothDeviceDiscoveryAgent(this);
+			connect(discoveryAgent, &QBluetoothDeviceDiscoveryAgent::deviceDiscovered, this, &BTDiscovery::btDeviceDiscovered);
+		}
 		qDebug() << "starting BLE discovery";
 		discoveryAgent->start();
-#endif
-#if defined(Q_OS_ANDROID)
+#else
 		getBluetoothDevices();
 		// and add the paired devices to the internal data
 		// So behaviour is same on Linux/Bluez stack and
 		// Android/Java stack with respect to discovery
-		for (int i = 0; i < btPairedDevices.length(); i++) {
+		for (int i = 0; i < btPairedDevices.length(); i++)
 			btDeviceDiscoveredMain(btPairedDevices[i]);
-
-		}
 #endif
-		for (int i = 0; i < btPairedDevices.length(); i++) {
+		for (int i = 0; i < btPairedDevices.length(); i++)
 			qDebug() << "Paired =" << btPairedDevices[i].name << btPairedDevices[i].address;
-		}
+
 #if defined(Q_OS_IOS) || (defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID))
 		QTimer timer;
 		timer.setSingleShot(true);
@@ -176,12 +187,11 @@ void BTDiscovery::btDeviceDiscovered(const QBluetoothDeviceInfo &device)
 		qDebug() << id.toByteArray();
 	}
 
-#if defined(Q_OS_IOS)
-	// On Desktop this is called when "Save" button is clicked. All
-	// DeviceInfo are stored as data on the ui list items.
-	// On mobile (iOS) the current ConnectionListModel does not support
-	// additional data, so just save all discovered devices.
-
+#if defined(Q_OS_IOS) || defined(Q_OS_MACOS) || defined(Q_OS_WIN)
+	// on Windows, macOS and iOS we need to scan in order to be able to access a device;
+	// let's remember the information we scanned on this run so we can at least
+	// refer back to it and don't need to open the separate scanning dialog every
+	// time we try to download from a BT/BLE dive computer.
 	saveBtDeviceInfo(btDeviceAddress(&device, false), device);
 #endif
 
@@ -298,7 +308,57 @@ bool BTDiscovery::checkException(const char* method, const QAndroidJniObject *ob
 }
 #endif // Q_OS_ANDROID
 
-QHash<QString, QBluetoothDeviceInfo> btDeviceInfo;
+void BTDiscovery::discoverAddress(QString address)
+{
+	// let's make sure there is no device name mixed in with the address
+	QString btAddress;
+	btAddress = extractBluetoothAddress(address);
+
+#if defined(Q_OS_MACOS)
+	// macOS appears to need a fresh scan if we want to switch devices
+	static QString lastAddress;
+	if (lastAddress != address) {
+		btDeviceInfo.clear();
+		discoveryAgent->stop();
+		lastAddress = address;
+	}
+#endif
+	if (!btDeviceInfo.keys().contains(address) && !discoveryAgent->isActive()) {
+		qDebug() << "restarting discovery agent";
+		discoveryAgent->start();
+	}
+}
+
+bool isBluetoothAddress(const QString &address)
+{
+	return extractBluetoothAddress(address) != QString();
+}
+QString extractBluetoothAddress(const QString &address)
+{
+	QRegularExpression re("(LE:)*([0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}|{[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}})",
+			      QRegularExpression::CaseInsensitiveOption);
+	QRegularExpressionMatch m = re.match(address);
+	return m.captured(0);
+}
+
+QString extractBluetoothNameAddress(const QString &address, QString &name)
+{
+	// sometimes our device text is of the form "name (address)", sometimes it's just "address"
+	// let's simply return the address
+	name = QString();
+	QString extractedAddress = extractBluetoothAddress(address);
+	if (extractedAddress == address.trimmed())
+		return address;
+
+	QRegularExpression re("^([^()]+)\\(([^)]*\\))$");
+	QRegularExpressionMatch m = re.match(address);
+	if (m.hasMatch()) {
+		name = m.captured(1).trimmed();
+		return extractedAddress;
+	}
+	qDebug() << "can't parse address" << address;
+	return QString();
+}
 
 void saveBtDeviceInfo(const QString &devaddr, QBluetoothDeviceInfo deviceInfo)
 {
@@ -309,7 +369,20 @@ QBluetoothDeviceInfo getBtDeviceInfo(const QString &devaddr)
 {
 	if (btDeviceInfo.contains(devaddr))
 		return btDeviceInfo[devaddr];
-	qDebug() << "need to scan for" << devaddr;
+	if(!btDeviceInfo.keys().contains(devaddr)) {
+		qDebug() << "still looking scan is still running, we should just wait for a few moments";
+		// wait for a maximum of 30 more seconds
+		// yes, that seems crazy, but on my Mac I see this take more than 20 seconds
+		QElapsedTimer timer;
+		timer.start();
+		do {
+			if (btDeviceInfo.keys().contains(devaddr))
+				return btDeviceInfo[devaddr];
+			QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
+			QThread::msleep(100);
+		} while (timer.elapsed() < 30000);
+	}
+	qDebug() << "notify user that we can't find" << devaddr;
 	return QBluetoothDeviceInfo();
 }
 #endif // BT_SUPPORT
