@@ -540,6 +540,8 @@ void clear_vpmb_state(struct deco_state *ds) {
 void clear_deco(struct deco_state *ds, double surface_pressure)
 {
 	int ci;
+
+	memset(ds, 0, sizeof(*ds));
 	clear_vpmb_state(ds);
 	for (ci = 0; ci < 16; ci++) {
 		ds->tissue_n2_sat[ci] = (surface_pressure - ((in_planner() && (decoMode() == VPMB)) ? WV_PRESSURE_SCHREINER : WV_PRESSURE)) * N2_IN_AIR / 1000;
@@ -551,6 +553,7 @@ void clear_deco(struct deco_state *ds, double surface_pressure)
 	}
 	ds->gf_low_pressure_this_dive = surface_pressure + buehlmann_config.gf_low_position_min;
 	ds->max_ambient_pressure = 0.0;
+	ds->ci_pointing_to_guiding_tissue = -1;
 }
 
 void cache_deco_state(struct deco_state *src, struct deco_state **cached_datap)
