@@ -292,10 +292,10 @@ typedef struct dive_trip
 	char *location;
 	char *notes;
 	struct dive *dives;
-	int nrdives;
+	int nrdives, showndives;
 	/* Used by the io-routines to mark trips that have already been written. */
 	bool saved;
-	unsigned expanded : 1, selected : 1, autogen : 1, fixup : 1;
+	bool autogen;
 	struct dive_trip *next;
 } dive_trip_t;
 
@@ -309,9 +309,8 @@ struct dive {
 	struct dive *next, **pprev;
 	bool selected;
 	bool hidden_by_filter;
-	bool downloaded;
 	timestamp_t when;
-	uint32_t dive_site_uuid;
+	struct dive_site *dive_site;
 	char *notes;
 	char *divemaster, *buddy;
 	int rating;
@@ -364,8 +363,7 @@ extern enum divemode_t get_divemode_at_time(const struct divecomputer *dc, int d
 struct picture {
 	char *filename;
 	offset_t offset;
-	degrees_t latitude;
-	degrees_t longitude;
+	location_t location;
 	struct picture *next;
 };
 
@@ -377,6 +375,7 @@ struct picture {
 	for (struct picture *picture = (_divestruct).picture_list; picture; picture = picture->next)
 
 extern struct picture *alloc_picture();
+extern void free_picture(struct picture *picture);
 extern bool dive_check_picture_time(const struct dive *d, int shift_time, timestamp_t timestamp);
 extern void dive_create_picture(struct dive *d, const char *filename, int shift_time, bool match_all);
 extern void dive_add_picture(struct dive *d, struct picture *newpic);
@@ -417,30 +416,30 @@ extern bool autogroup;
 
 extern void add_dive_to_trip(struct dive *, dive_trip_t *);
 
+struct dive *unregister_dive(int idx);
 extern void delete_single_dive(int idx);
+extern int dive_get_insertion_index(struct dive *dive);
 extern void add_single_dive(int idx, struct dive *dive);
 
 extern void insert_trip(dive_trip_t **trip);
-extern struct dive_trip *clone_empty_trip(struct dive_trip *trip);
-
+extern void insert_trip_dont_merge(dive_trip_t *trip);
+extern void unregister_trip(dive_trip_t *trip);
+extern void free_trip(dive_trip_t *trip);
 
 extern const struct units SI_units, IMPERIAL_units;
-extern struct units xml_parsing_units;
 
 extern const struct units *get_units(void);
 extern int run_survey, verbose, quit, force_root;
 
 struct dive_table {
-	int nr, allocated, preexisting;
+	int nr, allocated;
 	struct dive **dives;
 };
 
 extern struct dive_table dive_table, downloadTable;
 extern struct dive displayed_dive;
-extern struct dive_site displayed_dive_site;
-extern int selected_dive;
 extern unsigned int dc_number;
-#define current_dive (get_dive(selected_dive))
+extern struct dive *current_dive;
 #define current_dc (get_dive_dc(current_dive, dc_number))
 #define displayed_dc (get_dive_dc(&displayed_dive, dc_number))
 
@@ -544,6 +543,7 @@ extern timestamp_t utc_mktime(struct tm *tm);
 extern void utc_mkdate(timestamp_t, struct tm *tm);
 
 extern struct dive *alloc_dive(void);
+extern void free_dive(struct dive *);
 extern void record_dive_to_table(struct dive *dive, struct dive_table *table);
 extern void record_dive(struct dive *dive);
 extern void clear_dive(struct dive *dive);
@@ -551,8 +551,6 @@ extern void copy_dive(const struct dive *s, struct dive *d);
 extern void selective_copy_dive(const struct dive *s, struct dive *d, struct dive_components what, bool clear);
 extern struct dive *clone_dive(struct dive *s);
 
-extern void clear_table(struct dive_table *table);
-
 extern void alloc_samples(struct divecomputer *dc, int num);
 extern void free_samples(struct divecomputer *dc);
 extern struct sample *prepare_sample(struct divecomputer *dc);
@@ -561,15 +559,18 @@ extern struct sample *add_sample(const struct sample *sample, int time, struct d
 extern void add_sample_pressure(struct sample *sample, int sensor, int mbar);
 extern int legacy_format_o2pressures(const struct dive *dive, const struct divecomputer *dc);
 
+extern bool dive_less_than(const struct dive *a, const struct dive *b);
 extern void sort_table(struct dive_table *table);
 extern struct dive *fixup_dive(struct dive *dive);
 extern void fixup_dc_duration(struct divecomputer *dc);
 extern int dive_getUniqID();
 extern unsigned int dc_airtemp(const struct divecomputer *dc);
 extern unsigned int dc_watertemp(const struct divecomputer *dc);
-extern int split_dive(struct dive *);
+extern int split_dive_dont_insert(const struct dive *dive, struct dive **new1, struct dive **new2);
+extern void split_dive(struct dive *);
+extern int split_dive_at_time_dont_insert(const struct dive *dive, duration_t time, struct dive **new1, struct dive **new2);
 extern void split_dive_at_time(struct dive *dive, duration_t time);
-extern struct dive *merge_dives(struct dive *a, struct dive *b, int offset, bool prefer_downloaded);
+extern struct dive *merge_dives(const struct dive *a, const struct dive *b, int offset, bool prefer_downloaded, struct dive_trip **trip);
 extern struct dive *try_to_merge(struct dive *a, struct dive *b, bool prefer_downloaded);
 extern struct event *clone_event(const struct event *src_ev);
 extern void copy_events(const struct divecomputer *s, struct divecomputer *d);
