@@ -11,56 +11,49 @@
 struct dive_site_table dive_site_table;
 
 /* there could be multiple sites of the same name - return the first one */
-uint32_t get_dive_site_uuid_by_name(const char *name, struct dive_site **dsp)
+struct dive_site *get_dive_site_by_name(const char *name)
 {
 	int i;
 	struct dive_site *ds;
 	for_each_dive_site (i, ds) {
-		if (same_string(ds->name, name)) {
-			if (dsp)
-				*dsp = ds;
-			return ds->uuid;
-		}
+		if (same_string(ds->name, name))
+			return ds;
 	}
-	return 0;
+	return NULL;
 }
 
 /* there could be multiple sites at the same GPS fix - return the first one */
-uint32_t get_dive_site_uuid_by_gps(degrees_t latitude, degrees_t longitude, struct dive_site **dsp)
+struct dive_site *get_dive_site_by_gps(const location_t *loc)
 {
 	int i;
 	struct dive_site *ds;
 	for_each_dive_site (i, ds) {
-		if (ds->latitude.udeg == latitude.udeg && ds->longitude.udeg == longitude.udeg) {
-			if (dsp)
-				*dsp = ds;
-			return ds->uuid;
-		}
+		if (same_location(loc, &ds->location))
+			return ds;
 	}
-	return 0;
+	return NULL;
 }
 
-
 /* to avoid a bug where we have two dive sites with different name and the same GPS coordinates
  * and first get the gps coordinates (reading a V2 file) and happen to get back "the other" name,
  * this function allows us to verify if a very specific name/GPS combination already exists */
-uint32_t get_dive_site_uuid_by_gps_and_name(char *name, degrees_t latitude, degrees_t longitude)
+struct dive_site *get_dive_site_by_gps_and_name(char *name, const location_t *loc)
 {
 	int i;
 	struct dive_site *ds;
 	for_each_dive_site (i, ds) {
-		if (ds->latitude.udeg == latitude.udeg && ds->longitude.udeg == longitude.udeg && same_string(ds->name, name))
-			return ds->uuid;
+		if (same_location(loc, &ds->location) && same_string(ds->name, name))
+			return ds;
 	}
-	return 0;
+	return NULL;
 }
 
 // Calculate the distance in meters between two coordinates.
-unsigned int get_distance(degrees_t lat1, degrees_t lon1, degrees_t lat2, degrees_t lon2)
+unsigned int get_distance(const location_t *loc1, const location_t *loc2)
 {
-	double lat2_r = udeg_to_radians(lat2.udeg);
-	double lat_d_r = udeg_to_radians(lat2.udeg-lat1.udeg);
-	double lon_d_r = udeg_to_radians(lon2.udeg-lon1.udeg);
+	double lat2_r = udeg_to_radians(loc2->lat.udeg);
+	double lat_d_r = udeg_to_radians(loc2->lat.udeg - loc1->lat.udeg);
+	double lon_d_r = udeg_to_radians(loc2->lon.udeg - loc1->lon.udeg);
 
 	double a = sin(lat_d_r/2) * sin(lat_d_r/2) +
 		cos(lat2_r) * cos(lat2_r) * sin(lon_d_r/2) * sin(lon_d_r/2);
@@ -71,22 +64,19 @@ unsigned int get_distance(degrees_t lat1, degrees_t lon1, degrees_t lat2, degree
 }
 
 /* find the closest one, no more than distance meters away - if more than one at same distance, pick the first */
-uint32_t get_dive_site_uuid_by_gps_proximity(degrees_t latitude, degrees_t longitude, int distance, struct dive_site **dsp)
+struct dive_site *get_dive_site_by_gps_proximity(const location_t *loc, int distance)
 {
 	int i;
-	int uuid = 0;
-	struct dive_site *ds;
+	struct dive_site *ds, *res = NULL;
 	unsigned int cur_distance, min_distance = distance;
 	for_each_dive_site (i, ds) {
 		if (dive_site_has_gps_location(ds) &&
-		    (cur_distance = get_distance(ds->latitude, ds->longitude, latitude, longitude)) < min_distance) {
+		    (cur_distance = get_distance(&ds->location, loc)) < min_distance) {
 			min_distance = cur_distance;
-			uuid = ds->uuid;
-			if (dsp)
-				*dsp = ds;
+			res = ds;
 		}
 	}
-	return uuid;
+	return res;
 }
 
 /* try to create a uniqe ID - fingers crossed */
@@ -140,26 +130,30 @@ struct dive_site *alloc_or_get_dive_site(uint32_t uuid)
 	return ds;
 }
 
-int nr_of_dives_at_dive_site(uint32_t uuid, bool select_only)
+int nr_of_dives_at_dive_site(struct dive_site *ds, bool select_only)
 {
 	int j;
 	int nr = 0;
 	struct dive *d;
+	if (!ds)
+		return 0;
 	for_each_dive(j, d) {
-		if (d->dive_site_uuid == uuid && (!select_only || d->selected)) {
+		if (d->dive_site == ds && (!select_only || d->selected)) {
 			nr++;
 		}
 	}
 	return nr;
 }
 
-bool is_dive_site_used(uint32_t uuid, bool select_only)
+bool is_dive_site_used(struct dive_site *ds, bool select_only)
 {
 	int j;
 	bool found = false;
 	struct dive *d;
+	if (!ds)
+		return false;
 	for_each_dive(j, d) {
-		if (d->dive_site_uuid == uuid && (!select_only || d->selected)) {
+		if (d->dive_site == ds && (!select_only || d->selected)) {
 			found = true;
 			break;
 		}
@@ -167,15 +161,23 @@ bool is_dive_site_used(uint32_t uuid, bool select_only)
 	return found;
 }
 
-void delete_dive_site(uint32_t id)
+void free_dive_site(struct dive_site *ds)
+{
+	if (ds) {
+		free(ds->name);
+		free(ds->notes);
+		free(ds->description);
+		free_taxonomy(&ds->taxonomy);
+		free(ds);
+	}
+}
+
+void delete_dive_site(struct dive_site *ds)
 {
 	int nr = dive_site_table.nr;
 	for (int i = 0; i < nr; i++) {
-		struct dive_site *ds = get_dive_site(i);
-		if (ds->uuid == id) {
-			free(ds->name);
-			free(ds->notes);
-			free(ds);
+		if (ds == get_dive_site(i)) {
+			free_dive_site(ds);
 			if (nr - 1 > i)
 				memmove(&dive_site_table.dive_sites[i],
 					&dive_site_table.dive_sites[i+1],
@@ -204,16 +206,16 @@ uint32_t create_divesite_uuid(const char *name, timestamp_t divetime)
 }
 
 /* allocate a new site and add it to the table */
-uint32_t create_dive_site(const char *name, timestamp_t divetime)
+struct dive_site *create_dive_site(const char *name, timestamp_t divetime)
 {
 	uint32_t uuid = create_divesite_uuid(name, divetime);
 	struct dive_site *ds = alloc_or_get_dive_site(uuid);
 	ds->name = copy_string(name);
-	return uuid;
+	return ds;
 }
 
 /* same as before, but with current time if no current_dive is present */
-uint32_t create_dive_site_from_current_dive(const char *name)
+struct dive_site *create_dive_site_from_current_dive(const char *name)
 {
 	if (current_dive != NULL) {
 		return create_dive_site(name, current_dive->when);
@@ -226,15 +228,14 @@ uint32_t create_dive_site_from_current_dive(const char *name)
 }
 
 /* same as before, but with GPS data */
-uint32_t create_dive_site_with_gps(const char *name, degrees_t latitude, degrees_t longitude, timestamp_t divetime)
+struct dive_site *create_dive_site_with_gps(const char *name, const location_t *loc, timestamp_t divetime)
 {
 	uint32_t uuid = create_divesite_uuid(name, divetime);
 	struct dive_site *ds = alloc_or_get_dive_site(uuid);
 	ds->name = copy_string(name);
-	ds->latitude = latitude;
-	ds->longitude = longitude;
+	ds->location = *loc;
 
-	return ds->uuid;
+	return ds;
 }
 
 /* a uuid is always present - but if all the other fields are empty, the dive site is pointless */
@@ -244,43 +245,21 @@ bool dive_site_is_empty(struct dive_site *ds)
 	       (empty_string(ds->name) &&
 	       empty_string(ds->description) &&
 	       empty_string(ds->notes) &&
-	       ds->latitude.udeg == 0 &&
-	       ds->longitude.udeg == 0);
+	       !has_location(&ds->location));
 }
 
-void copy_dive_site_taxonomy(struct dive_site *orig, struct dive_site *copy)
-{
-	if (orig->taxonomy.category == NULL) {
-		free_taxonomy(&copy->taxonomy);
-	} else {
-		if (copy->taxonomy.category == NULL)
-			copy->taxonomy.category = alloc_taxonomy();
-		for (int i = 0; i < TC_NR_CATEGORIES; i++) {
-			if (i < copy->taxonomy.nr) {
-				free((void *)copy->taxonomy.category[i].value);
-				copy->taxonomy.category[i].value = NULL;
-			}
-			if (i < orig->taxonomy.nr) {
-				copy->taxonomy.category[i] = orig->taxonomy.category[i];
-				copy->taxonomy.category[i].value = copy_string(orig->taxonomy.category[i].value);
-			}
-		}
-		copy->taxonomy.nr = orig->taxonomy.nr;
-	}
-}
 void copy_dive_site(struct dive_site *orig, struct dive_site *copy)
 {
 	free(copy->name);
 	free(copy->notes);
 	free(copy->description);
 
-	copy->latitude = orig->latitude;
-	copy->longitude = orig->longitude;
+	copy->location = orig->location;
 	copy->name = copy_string(orig->name);
 	copy->notes = copy_string(orig->notes);
 	copy->description = copy_string(orig->description);
 	copy->uuid = orig->uuid;
-	copy_dive_site_taxonomy(orig, copy);
+	copy_taxonomy(&orig->taxonomy, &copy->taxonomy);
 }
 
 static void merge_string(char **a, char **b)
@@ -304,8 +283,7 @@ static void merge_string(char **a, char **b)
 
 void merge_dive_site(struct dive_site *a, struct dive_site *b)
 {
-	if (!a->latitude.udeg) a->latitude.udeg = b->latitude.udeg;
-	if (!a->longitude.udeg) a->longitude.udeg = b->longitude.udeg;
+	if (!has_location(&a->location)) a->location = b->location;
 	merge_string(&a->name, &b->name);
 	merge_string(&a->notes, &b->notes);
 	merge_string(&a->description, &b->description);
@@ -324,38 +302,38 @@ void clear_dive_site(struct dive_site *ds)
 	ds->name = NULL;
 	ds->notes = NULL;
 	ds->description = NULL;
-	ds->latitude.udeg = 0;
-	ds->longitude.udeg = 0;
+	ds->location.lat.udeg = 0;
+	ds->location.lon.udeg = 0;
 	ds->uuid = 0;
 	ds->taxonomy.nr = 0;
 	free_taxonomy(&ds->taxonomy);
 }
 
-void merge_dive_sites(uint32_t ref, uint32_t* uuids, int count)
+void merge_dive_sites(struct dive_site *ref, struct dive_site *dive_sites[], int count)
 {
 	int curr_dive, i;
 	struct dive *d;
 	for(i = 0; i < count; i++){
-		if (uuids[i] == ref)
+		if (dive_sites[i] == ref)
 			continue;
 
 		for_each_dive(curr_dive, d) {
-			if (d->dive_site_uuid != uuids[i] )
+			if (d->dive_site != dive_sites[i] )
 				continue;
-			d->dive_site_uuid = ref;
+			d->dive_site = ref;
 			invalidate_dive_cache(d);
 		}
 	}
 
 	for(i = 0; i < count; i++) {
-		if (uuids[i] == ref)
+		if (dive_sites[i] == ref)
 			continue;
-		delete_dive_site(uuids[i]);
+		delete_dive_site(dive_sites[i]);
 	}
 	mark_divelist_changed(true);
 }
 
-uint32_t find_or_create_dive_site_with_name(const char *name, timestamp_t divetime)
+struct dive_site *find_or_create_dive_site_with_name(const char *name, timestamp_t divetime)
 {
 	int i;
 	struct dive_site *ds;
@@ -364,7 +342,7 @@ uint32_t find_or_create_dive_site_with_name(const char *name, timestamp_t diveti
 			break;
 	}
 	if (ds)
-		return ds->uuid;
+		return ds;
 	return create_dive_site(name, divetime);
 }
 
