@@ -7,8 +7,8 @@ struct memblock {
 	size_t size;
 };
 
-extern int try_to_open_cochran(const char *filename, struct memblock *mem);
-extern int try_to_open_liquivision(const char *filename, struct memblock *mem);
+extern int try_to_open_cochran(const char *filename, struct memblock *mem, struct dive_table *table);
+extern int try_to_open_liquivision(const char *filename, struct memblock *mem, struct dive_table *table);
 extern int datatrak_import(struct memblock *mem, struct dive_table *table);
 extern void ostctools_import(const char *file, struct dive_table *table);
 
