@@ -92,12 +92,31 @@ void BLEObject::characteristicWritten(const QLowEnergyCharacteristic &c, const Q
 void BLEObject::writeCompleted(const QLowEnergyDescriptor&, const QByteArray&)
 {
 	qDebug() << "BLE write completed";
+	desc_written++;
 }
 
 void BLEObject::addService(const QBluetoothUuid &newService)
 {
 	qDebug() << "Found service" << newService;
 
+	if (IS_HW(device)) {
+		/* The HW BT/BLE piece or hardware uses, what we
+		 * call here, "a Standard UUID. It is standard because the Telit/Stollmann
+		 * manufacturer applied for an own UUID for its product, and this was granted
+		 * by the Bluetooth SIG.
+		 */
+		if (newService != QUuid("{0000fefb-0000-1000-8000-00805f9b34fb}"))
+			return; // skip all services except the right one
+	} else {
+		bool isStandardUuid = false;
+
+		newService.toUInt16(&isStandardUuid);
+		if (isStandardUuid) {
+			qDebug () << " .. ignoring standard service" << newService;
+			return;
+		}
+	}
+
 	auto service = controller->createServiceObject(newService, this);
 	qDebug() << " .. created service object" << service;
 	if (service) {
@@ -112,6 +131,7 @@ BLEObject::BLEObject(QLowEnergyController *c, dc_user_device_t *d)
 	device = d;
 	debugCounter = 0;
 	isCharacteristicWritten = false;
+	timeout = BLE_TIMEOUT;
 }
 
 BLEObject::~BLEObject()
@@ -184,7 +204,7 @@ dc_status_t BLEObject::read(void *data, size_t size, size_t *actual)
 
 		qDebug() << QTime::currentTime() << "packet WAIT";
 
-		WAITFOR(!receivedPackets.isEmpty(), BLE_TIMEOUT);
+		WAITFOR(!receivedPackets.isEmpty(), timeout);
 		if (receivedPackets.isEmpty())
 			return DC_STATUS_IO;
 	}
@@ -248,39 +268,23 @@ dc_status_t BLEObject::select_preferred_service(void)
 		if (s->state() != QLowEnergyService::ServiceDiscovered)
 			continue;
 
-		bool isStandardUuid = false;
+		bool hasread = false;
+		bool haswrite = false;
 		QBluetoothUuid uuid = s->serviceUuid();
 
-		uuid.toUInt16(&isStandardUuid);
+		foreach (const QLowEnergyCharacteristic &c, s->characteristics()) {
+			hasread |= is_read_characteristic(c);
+			haswrite |= is_write_characteristic(c);
+		}
 
-		if (IS_HW(device)) {
-			/* The HW BT/BLE piece or hardware uses, what we
-			 * call here, "a Standard UUID. It is standard because the Telit/Stollmann
-			 * manufacturer applied for an own UUID for its product, and this was granted
-			 * by the Bluetooth SIG.
-			 */
-			if (uuid != QUuid("{0000fefb-0000-1000-8000-00805f9b34fb}"))
-				continue; // skip all services except the right one
-		} else if (isStandardUuid) {
-			qDebug () << " .. ignoring standard service" << uuid;
+		if (!hasread) {
+			qDebug () << " .. ignoring service without read characteristic" << uuid;
 			continue;
-		} else {
-			bool hasread = false;
-			bool haswrite = false;
+		}
 
-			foreach (const QLowEnergyCharacteristic &c, s->characteristics()) {
-				hasread |= is_read_characteristic(c);
-				haswrite |= is_write_characteristic(c);
-			}
-
-			if (!hasread) {
-				qDebug () << " .. ignoring service without read characteristic" << uuid;
-				continue;
-			}
-			if (!haswrite) {
-				qDebug () << " .. ignoring service without write characteristic" << uuid;
-				continue;
-			}
+		if (!haswrite) {
+			qDebug () << " .. ignoring service without write characteristic" << uuid;
+			continue;
 		}
 
 		// We now know that the service has both read and write characteristics
@@ -366,6 +370,7 @@ dc_status_t BLEObject::setupHwTerminalIo(QList<QLowEnergyCharacteristic> allC)
 	return setHwCredit(MAXIMAL_HW_CREDIT);
 }
 
+#if !defined(Q_OS_WIN)
 // Bluez is broken, and doesn't have a sane way to query
 // whether to use a random address or not. So we have to
 // fake it.
@@ -373,6 +378,7 @@ static int use_random_address(dc_user_device_t *user_device)
 {
 	return IS_SHEARWATER(user_device) || IS_GARMIN(user_device);
 }
+#endif
 
 dc_status_t qt_ble_open(void **io, dc_context_t *, const char *devaddr, dc_user_device_t *user_device)
 {
@@ -405,8 +411,10 @@ dc_status_t qt_ble_open(void **io, dc_context_t *, const char *devaddr, dc_user_
 #endif
 	qDebug() << "qt_ble_open(" << devaddr << ")";
 
+#if !defined(Q_OS_WIN)
 	if (use_random_address(user_device))
 		controller->setRemoteAddressType(QLowEnergyController::RandomAddress);
+#endif
 
 	// Try to connect to the device
 	controller->connectToDevice();
@@ -483,6 +491,7 @@ dc_status_t qt_ble_open(void **io, dc_context_t *, const char *devaddr, dc_user_
 			qDebug() << "now writing \"0x0100\" to the descriptor" << d.uuid().toString();
 
 			ble->preferredService()->writeDescriptor(d, QByteArray::fromHex("0100"));
+			WAITFOR(ble->descriptorWritten(), 1000);
 			break;
 		}
 	}
@@ -508,6 +517,17 @@ static void checkThreshold()
 	}
 }
 
+/*
+ * NOTE! The 'set_timeout()' function only affects the timeout
+ * for qt_ble_read(), not for the various general BLE operations.
+ */
+dc_status_t qt_ble_set_timeout(void *io, int timeout)
+{
+	BLEObject *ble = (BLEObject *) io;
+	ble->set_timeout(timeout);
+	return DC_STATUS_SUCCESS;
+}
+
 dc_status_t qt_ble_read(void *io, void* data, size_t size, size_t *actual)
 {
 	checkThreshold();
@@ -522,4 +542,10 @@ dc_status_t qt_ble_write(void *io, const void* data, size_t size, size_t *actual
 	return ble->write(data, size, actual);
 }
 
+const char *qt_ble_get_name(void *io)
+{
+	BLEObject *ble = (BLEObject *) io;
+	return ble->get_name();
+}
+
 } /* extern "C" */
