@@ -23,9 +23,6 @@
 #include <QDebug>
 #include <QStandardPaths>
 #include <QJsonDocument>
-#include <QNetworkReply>
-#include <QNetworkRequest>
-#include <QNetworkAccessManager>
 #include <QNetworkProxy>
 #include <QDateTime>
 #include <QImageReader>
@@ -75,14 +72,16 @@ QString distance_string(int distanceInMeters)
 	return str;
 }
 
-extern "C" const char *printGPSCoords(int lat, int lon)
+extern "C" const char *printGPSCoords(const location_t *loc)
 {
+	int lat = loc->lat.udeg;
+	int lon = loc->lon.udeg;
 	unsigned int latdeg, londeg;
 	unsigned int latmin, lonmin;
 	double latsec, lonsec;
 	QString lath, lonh, result;
 
-	if (!lat && !lon)
+	if (!has_location(loc))
 		return strdup("");
 
 	if (prefs.coordinates_traditional) {
@@ -247,30 +246,24 @@ bool parseGpsText(const QString &gps_text, double *latitude, double *longitude)
 #if 0 // we'll need something like this for the dive site management, eventually
 bool gpsHasChanged(struct dive *dive, struct dive *master, const QString &gps_text, bool *parsed_out)
 {
-	double latitude, longitude;
-	int latudeg, longudeg;
+	location_t location;
 	bool ignore;
 	bool *parsed = parsed_out ?: &ignore;
 	*parsed = true;
 
 	/* if we have a master and the dive's gps address is different from it,
 	 * don't change the dive */
-	if (master && (master->latitude.udeg != dive->latitude.udeg ||
-		       master->longitude.udeg != dive->longitude.udeg))
+	if (master && !same_location(&master->location, &dive->location))
 		return false;
 
-	if (!(*parsed = parseGpsText(gps_text, &latitude, &longitude)))
+	if (!(*parsed = parseGpsText(gps_text, location)))
 		return false;
 
-	latudeg = lrint(1000000 * latitude);
-	longudeg = lrint(1000000 * longitude);
-
 	/* if dive gps didn't change, nothing changed */
-	if (dive->latitude.udeg == latudeg && dive->longitude.udeg == longudeg)
+	if (same_location(&dive->location, location))
 		return false;
 	/* ok, update the dive and mark things changed */
-	dive->latitude.udeg = latudeg;
-	dive->longitude.udeg = longudeg;
+	dive->location = location;
 	return true;
 }
 #endif
@@ -991,31 +984,6 @@ QString get_trip_date_string(timestamp_t when, int nr, bool getday)
 
 }
 
-extern "C" void reverseGeoLookup(degrees_t latitude, degrees_t longitude, uint32_t uuid)
-{
-	QNetworkRequest request;
-	QNetworkAccessManager *rgl = new QNetworkAccessManager();
-	request.setUrl(QString("http://open.mapquestapi.com/nominatim/v1/reverse.php?format=json&accept-language=%1&lat=%2&lon=%3")
-		       .arg(uiLanguage(NULL)).arg(latitude.udeg / 1000000.0).arg(longitude.udeg / 1000000.0));
-	request.setRawHeader("Accept", "text/json");
-	request.setRawHeader("User-Agent", getUserAgent().toUtf8());
-	QNetworkReply *reply = rgl->get(request);
-	QEventLoop loop;
-	QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
-	loop.exec();
-	QJsonParseError errorObject;
-	QJsonDocument jsonDoc = QJsonDocument::fromJson(reply->readAll(), &errorObject);
-	if (errorObject.error != QJsonParseError::NoError) {
-		qDebug() << errorObject.errorString();
-	} else {
-		QJsonObject obj = jsonDoc.object();
-		QJsonObject address = obj.value("address").toObject();
-		qDebug() << "found country:" << address.value("country").toString();
-		struct dive_site *ds = get_dive_site_by_uuid(uuid);
-		ds->notes = add_to_string(ds->notes, "countrytag: %s", qPrintable(address.value("country").toString()));
-	}
-}
-
 static QMutex hashOfMutex;
 static QHash<QString, QString> localFilenameOf;
 
