@@ -38,14 +38,6 @@ static void cond_put_format(int cond, struct membuffer *b, const char *fmt, ...)
 
 #define SAVE(str, x) cond_put_format(dive->x, b, str " %d\n", dive->x)
 
-static void show_gps(struct membuffer *b, degrees_t latitude, degrees_t longitude)
-{
-	if (latitude.udeg || longitude.udeg) {
-		put_degrees(b, latitude, "gps ", " ");
-		put_degrees(b, longitude, "", "\n");
-	}
-}
-
 static void quote(struct membuffer *b, const char *text)
 {
 	const char *p = text;
@@ -439,17 +431,16 @@ static void create_dive_buffer(struct dive *dive, struct membuffer *b)
 	SAVE("visibility", visibility);
 	cond_put_format(dive->tripflag == NO_TRIP, b, "notrip\n");
 	save_tags(b, dive->tag_list);
-	cond_put_format(dive->dive_site_uuid && get_dive_site_by_uuid(dive->dive_site_uuid),
-			b, "divesiteid %08x\n", dive->dive_site_uuid);
-	if (verbose && dive->dive_site_uuid && !get_dive_site_by_uuid(dive->dive_site_uuid))
-		fprintf(stderr, "removed reference to non-existant dive site with uuid %08x\n", dive->dive_site_uuid);
+	if (dive->dive_site)
+		put_format(b, "divesiteid %08x\n", dive->dive_site->uuid);
+	if (verbose && dive->dive_site)
+		fprintf(stderr, "removed reference to non-existant dive site with uuid %08x\n", dive->dive_site->uuid);
 	save_overview(b, dive);
 	save_cylinder_info(b, dive);
 	save_weightsystem_info(b, dive);
 	save_dive_temperature(b, dive);
 }
 
-
 /*
  * libgit2 has a "git_treebuilder" concept, but it's broken, and can not
  * be used to do a flat tree (like the git "index") nor a recursive tree.
@@ -613,7 +604,7 @@ static int save_one_picture(git_repository *repo, struct dir *dir, struct pictur
 	unsigned h;
 
 	show_utf8(&buf, "filename ", pic->filename, "\n");
-	show_gps(&buf, pic->latitude, pic->longitude);
+	put_location(&buf, &pic->location, "gps ", "\n");
 
 	/* Picture loading will load even negative offsets.. */
 	if (offset < 0) {
@@ -891,14 +882,14 @@ static void save_divesites(git_repository *repo, struct dir *tree)
 	for (int i = 0; i < dive_site_table.nr; i++) {
 		struct membuffer b = { 0 };
 		struct dive_site *ds = get_dive_site(i);
-		if (dive_site_is_empty(ds) || !is_dive_site_used(ds->uuid, false)) {
+		if (dive_site_is_empty(ds) || !is_dive_site_used(ds, false)) {
 			int j;
 			struct dive *d;
 			for_each_dive(j, d) {
-				if (d->dive_site_uuid == ds->uuid)
-					d->dive_site_uuid = 0;
+				if (d->dive_site == ds)
+					d->dive_site = NULL;
 			}
-			delete_dive_site(ds->uuid);
+			delete_dive_site(ds);
 			i--; // since we just deleted that one
 			continue;
 		} else if (ds->name &&
@@ -908,10 +899,10 @@ static void save_divesites(git_repository *repo, struct dir *tree)
 			// these are the two default names for sites from
 			// the web service; if the site isn't used in any
 			// dive (really? you didn't rename it?), delete it
-			if (!is_dive_site_used(ds->uuid, false)) {
+			if (!is_dive_site_used(ds, false)) {
 				if (verbose)
 					fprintf(stderr, "Deleted unused auto-created dive site %s\n", ds->name);
-				delete_dive_site(ds->uuid);
+				delete_dive_site(ds);
 				i--; // since we just deleted that one
 				continue;
 			}
@@ -921,7 +912,7 @@ static void save_divesites(git_repository *repo, struct dir *tree)
 		show_utf8(&b, "name ", ds->name, "\n");
 		show_utf8(&b, "description ", ds->description, "\n");
 		show_utf8(&b, "notes ", ds->notes, "\n");
-		show_gps(&b, ds->latitude, ds->longitude);
+		put_location(&b, &ds->location, "gps ", "\n");
 		for (int j = 0; j < ds->taxonomy.nr; j++) {
 			struct taxonomy *t = &ds->taxonomy.category[j];
 			if (t->category != TC_NONE && t->value) {
