@@ -20,7 +20,7 @@ class DiveObjectHelper : public QObject {
 	Q_PROPERTY(QString location READ location CONSTANT)
 	Q_PROPERTY(QString gps READ gps CONSTANT)
 	Q_PROPERTY(QString gps_decimal READ gps_decimal CONSTANT)
-	Q_PROPERTY(QVariant dive_site_uuid READ dive_site_uuid CONSTANT)
+	Q_PROPERTY(QVariant dive_site READ dive_site CONSTANT)
 	Q_PROPERTY(QString duration READ duration CONSTANT)
 	Q_PROPERTY(bool noDive READ noDive CONSTANT)
 	Q_PROPERTY(QString depth READ depth CONSTANT)
@@ -49,11 +49,14 @@ class DiveObjectHelper : public QObject {
 	Q_PROPERTY(QStringList startPressure READ startPressure CONSTANT)
 	Q_PROPERTY(QStringList endPressure READ endPressure CONSTANT)
 	Q_PROPERTY(QStringList firstGas READ firstGas CONSTANT)
+	Q_PROPERTY(QString fullText READ fullText CONSTANT)
+	Q_PROPERTY(QString fullTextNoNotes READ fullTextNoNotes CONSTANT)
 public:
 	DiveObjectHelper(struct dive *dive = NULL);
 	~DiveObjectHelper();
 	int number() const;
 	int id() const;
+	struct dive *getDive() const;
 	int rating() const;
 	int visibility() const;
 	QString date() const;
@@ -62,7 +65,7 @@ public:
 	QString location() const;
 	QString gps() const;
 	QString gps_decimal() const;
-	QVariant dive_site_uuid() const;
+	QVariant dive_site() const;
 	QString duration() const;
 	bool noDive() const;
 	QString depth() const;
@@ -93,6 +96,8 @@ public:
 	QStringList startPressure() const;
 	QStringList endPressure() const;
 	QStringList firstGas() const;
+	QString fullText() const;
+	QString fullTextNoNotes() const;
 
 private:
 	struct dive *m_dive;
