@@ -100,6 +100,8 @@ struct preferences default_prefs = {
 	.auto_recalculate_thumbnails = true,
 	.extract_video_thumbnails = true,
 	.extract_video_thumbnails_position = 20,		// The first fifth seems like a reasonable place
+	.filterCaseSensitive = false,
+	.filterFullTextNotes = true,
 };
 
 int run_survey;
