@@ -35,8 +35,8 @@ void writeMarkers(struct membuffer *b, const bool selected_only)
 		struct dive_site *ds = get_dive_site_for_dive(dive);
 		if (!ds || !dive_site_has_gps_location(ds))
 			continue;
-		put_degrees(b, ds->latitude, "temp = new google.maps.Marker({position: new google.maps.LatLng(", "");
-		put_degrees(b, ds->longitude, ",", ")});\n");
+		put_degrees(b, ds->location.lat, "temp = new google.maps.Marker({position: new google.maps.LatLng(", "");
+		put_degrees(b, ds->location.lon, ",", ")});\n");
 		put_string(b, "markers.push(temp);\ntempinfowindow = new google.maps.InfoWindow({content: '<div id=\"content\">'+'<div id=\"siteNotice\">'+'</div>'+'<div id=\"bodyContent\">");
 		snprintf(pre, sizeof(pre), "<p>%s ", translate("gettextFromC", "Date:"));
 		put_HTML_date(b, dive, pre, "</p>");
