@@ -1,12 +1,14 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/divelogimportdialog.h"
 #include "desktop-widgets/mainwindow.h"
+#include "desktop-widgets/command.h"
 #include "core/color.h"
 #include "ui_divelogimportdialog.h"
 #include <QShortcut>
 #include <QDrag>
 #include <QMimeData>
 #include <QRegExp>
+#include <QUndoStack>
 #include "core/qthelper.h"
 #include "core/import-csv.h"
 
@@ -15,7 +17,24 @@ static QString subsurface_index = "subsurface/csvindex";
 
 #define SILENCE_WARNING 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ""
 
-const DiveLogImportDialog::CSVAppConfig DiveLogImportDialog::CSVApps[CSVAPPS] = {
+struct CSVAppConfig {
+	QString name;
+	int time;
+	int depth;
+	int temperature;
+	int po2;
+	int sensor1;
+	int sensor2;
+	int sensor3;
+	int cns;
+	int ndl;
+	int tts;
+	int stopdepth;
+	int pressure;
+	int setpoint;
+	QString separator;
+};
+static const CSVAppConfig CSVApps[] = {
 	// time, depth, temperature, po2, sensor1, sensor2, sensor3, cns, ndl, tts, stopdepth, pressure, setpoint
 	// indices are 0 based, -1 means the column doesn't exist
 	{ "Manual import", SILENCE_WARNING },
@@ -27,7 +46,7 @@ const DiveLogImportDialog::CSVAppConfig DiveLogImportDialog::CSVApps[CSVAPPS] =
 	{ "Seabear CSV", 0, 1, 5, -1, -1, -1, -1, -1, 2, 3, 4, 6, -1, ";" },
 	{ "SubsurfaceCSV", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, "Tab" },
 	{ "AV1", 0, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, " " },
-	{ NULL, SILENCE_WARNING }
+	{ "Poseidon MkVI", 0, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, "," },
 };
 
 enum Known {
@@ -39,7 +58,8 @@ enum Known {
 	SENSUS,
 	SEABEAR,
 	SUBSURFACE,
-	AV1
+	AV1,
+	POSEIDON
 };
 
 ColumnNameProvider::ColumnNameProvider(QObject *parent) : QAbstractListModel(parent)
@@ -107,8 +127,6 @@ int ColumnNameProvider::mymatch(QString value) const
 	return -1;
 }
 
-
-
 ColumnNameView::ColumnNameView(QWidget*)
 {
 	setAcceptDrops(true);
@@ -345,9 +363,10 @@ DiveLogImportDialog::DiveLogImportDialog(QStringList fn, QWidget *parent) : QDia
 	specialCSV << SUBSURFACE;
 	specialCSV << DL7;
 	specialCSV << AV1;
+	specialCSV << POSEIDON;
 
-	for (int i = 0; !CSVApps[i].name.isNull(); ++i)
-		ui->knownImports->addItem(CSVApps[i].name);
+	for (const CSVAppConfig &conf: CSVApps)
+		ui->knownImports->addItem(conf.name);
 
 	ui->CSVSeparator->addItems( QStringList() << tr("Tab") << "," << ";" << "|");
 
@@ -378,9 +397,24 @@ void DiveLogImportDialog::loadFileContentsKnownTypesSelected(int value)
 	loadFileContents(value, KNOWNTYPES);
 }
 
+// Turn a "*.csv" or "*.txt" filename into a pair of both, "*.csv" and "*.txt".
+// If the input wasn't either "*.csv" or "*.txt", then both returned strings
+// are empty
+static QPair<QString, QString> poseidonFileNames(const QString &fn)
+{
+	if (fn.endsWith(".csv", Qt::CaseInsensitive)) {
+		QString txt = fn.left(fn.size() - 3) + "txt";
+		return { fn, txt };
+	} else if (fn.endsWith(".txt", Qt::CaseInsensitive)) {
+		QString csv = fn.left(fn.size() - 3) + "csv";
+		return { csv, fn };
+	} else {
+		return { QString(), QString() };
+	}
+}
+
 void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 {
-	QFile f(fileNames.first());
 	QList<QStringList> fileColumns;
 	QStringList currColumns;
 	QStringList headers;
@@ -389,6 +423,7 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	bool xp5 = false;
 	bool apd = false;
 	bool dl7 = false;
+	bool poseidon = false;
 
 	// reset everything
 	ColumnNameProvider *provider = new ColumnNameProvider(this);
@@ -397,6 +432,26 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	resultModel = new ColumnNameResult(this);
 	ui->tableView->setModel(resultModel);
 
+	// Poseidon MkVI is special: it is made up of a .csv *and* a .txt file.
+	// If the user specified one, we'll try to check for the other.
+	QString fileName = fileNames.first();
+	QPair<QString, QString> pair = poseidonFileNames(fileName);
+	if (!pair.second.isEmpty()) {
+		QFile f_txt(pair.second);
+		f_txt.open(QFile::ReadOnly);
+		QString firstLine = f_txt.readLine();
+		if (firstLine.startsWith("MkVI_Config ")) {
+			poseidon = true;
+			fileName = pair.first; // Read data from CSV
+			headers.append("Time");
+			headers.append("Depth");
+			blockSignals(true);
+			ui->knownImports->setCurrentText("Poseidon MkVI");
+			blockSignals(false);
+		}
+	}
+
+	QFile f(fileName);
 	f.open(QFile::ReadOnly);
 	QString firstLine = f.readLine();
 	if (firstLine.contains("SEABEAR")) {
@@ -497,11 +552,8 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			}
 		}
 		firstLine = f.readLine().trimmed();
-
 	}
 
-
-
 	// Special handling for APD Log Viewer
 	if ((triggeredBy == KNOWNTYPES && (value == APD || value == APD2)) || (triggeredBy == INITIAL && fileNames.first().endsWith(".apd", Qt::CaseInsensitive))) {
 		QString apdseparator;
@@ -726,6 +778,12 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	while (rows < 10 && !f.atEnd()) {
 		QString currLine = f.readLine().trimmed();
 		currColumns = currLine.split(separator);
+		// For Poseidon, read only columns where the second value is 8 (=depth)
+		if (poseidon) {
+			if (currColumns.size() < 3 || currColumns[1] != "8")
+				continue;
+			currColumns.removeAt(1);
+		}
 		fileColumns.append(currColumns);
 		rows += 1;
 	}
@@ -840,11 +898,15 @@ int DiveLogImportDialog::parseTxtHeader(QString fileName, char **params, int pnr
 
 void DiveLogImportDialog::on_buttonBox_accepted()
 {
+	struct dive_table table = { 0 };
 	QStringList r = resultModel->result();
 	if (ui->knownImports->currentText() != "Manual import") {
 		for (int i = 0; i < fileNames.size(); ++i) {
 			if (ui->knownImports->currentText() == "Seabear CSV") {
-				parse_seabear_log(qPrintable(fileNames[i]), &dive_table);
+				parse_seabear_log(qPrintable(fileNames[i]), &table);
+			} else if (ui->knownImports->currentText() == "Poseidon MkVI") {
+				QPair<QString, QString> pair = poseidonFileNames(fileNames[i]);
+				parse_txt_file(qPrintable(pair.second), qPrintable(pair.first), &table);
 			} else {
 				char *params[49];
 				int pnr = 0;
@@ -861,7 +923,7 @@ void DiveLogImportDialog::on_buttonBox_accepted()
 				pnr = setup_csv_params(r, params, pnr);
 				parse_csv_file(qPrintable(fileNames[i]), params, pnr - 1,
 						specialCSV.contains(ui->knownImports->currentIndex()) ? qPrintable(CSVApps[ui->knownImports->currentIndex()].name) : "csv",
-						&dive_table);
+						&table);
 			}
 		}
 	} else {
@@ -925,7 +987,7 @@ void DiveLogImportDialog::on_buttonBox_accepted()
 				params[pnr++] = intdup(r.indexOf(tr("Rating")));
 				params[pnr++] = NULL;
 
-				parse_manual_file(qPrintable(fileNames[i]), params, pnr - 1, &dive_table);
+				parse_manual_file(qPrintable(fileNames[i]), params, pnr - 1, &table);
 			} else {
 				char *params[51];
 				int pnr = 0;
@@ -942,12 +1004,14 @@ void DiveLogImportDialog::on_buttonBox_accepted()
 				pnr = setup_csv_params(r, params, pnr);
 				parse_csv_file(qPrintable(fileNames[i]), params, pnr - 1,
 						specialCSV.contains(ui->knownImports->currentIndex()) ? qPrintable(CSVApps[ui->knownImports->currentIndex()].name) : "csv",
-						&dive_table);
+						&table);
 			}
 		}
 	}
 
-	process_imported_dives(false);
+	process_imported_dives(&table, false, false);
+	autogroup_dives();
+	Command::clear();
 	MainWindow::instance()->refreshDisplay();
 }
 
