@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/downloadfromdivecomputer.h"
+#include "desktop-widgets/command.h"
 #include "core/display.h"
 #include "core/qthelper.h"
 #include "core/settings/qPrefDiveComputer.h"
@@ -14,6 +15,7 @@
 #include <QMessageBox>
 #include <QShortcut>
 #include <QTimer>
+#include <QUndoStack>
 
 DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f),
 	downloading(false),
@@ -21,6 +23,9 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	timer(new QTimer(this)),
 	dumpWarningShown(false),
 	ostcFirmwareCheck(0),
+#if defined (BT_SUPPORT)
+	btd(nullptr),
+#endif
 	currentState(INITIAL)
 {
 	diveImportedModel = new DiveImportedModel(this);
@@ -89,7 +94,7 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	QString deviceText = qPrefDiveComputer::device();
 #if defined(BT_SUPPORT)
 	ui.bluetoothMode->setText(tr("Choose Bluetooth download mode"));
-	ui.bluetoothMode->setChecked(qPrefDiveComputer::download_mode() == DC_TRANSPORT_BLUETOOTH);
+	ui.bluetoothMode->setChecked(isBluetoothAddress(qPrefDiveComputer::device()));
 	btDeviceSelectionDialog = 0;
 	connect(ui.bluetoothMode, SIGNAL(stateChanged(int)), this, SLOT(enableBluetoothMode(int)));
 	connect(ui.chooseBluetoothDevice, SIGNAL(clicked()), this, SLOT(selectRemoteBluetoothDevice()));
@@ -165,19 +170,25 @@ void DownloadFromDCWidget::updateProgressBar()
 			progress_bar_text = "";
 	}
 	if (!empty_string(progress_bar_text)) {
+		// once the progress bar text is set, setup the maximum so the user sees actual progress
 		ui.progressBar->setFormat(progress_bar_text);
+		ui.progressBar->setMaximum(100);
 #if defined(Q_OS_MAC)
 		// on mac the progress bar doesn't show its text
 		ui.progressText->setText(progress_bar_text);
 #endif
 	} else {
 		if (IS_FP_SAME(progress_bar_fraction, 0.0)) {
+			// while we are waiting to connect, set the maximum to 0 so we get a busy indication
+			ui.progressBar->setMaximum(0);
 			ui.progressBar->setFormat(tr("Connecting to dive computer"));
 #if defined(Q_OS_MAC)
 			// on mac the progress bar doesn't show its text
 			ui.progressText->setText(tr("Connecting to dive computer"));
 #endif
 		} else {
+			// we have some progress - reset the maximum so the user sees actual progress
+			ui.progressBar->setMaximum(100);
 			ui.progressBar->setFormat("%p%");
 #if defined(Q_OS_MAC)
 			// on mac the progress bar doesn't show its text
@@ -299,6 +310,20 @@ void DownloadFromDCWidget::on_product_currentIndexChanged(const QString &)
 	updateDeviceEnabled();
 }
 
+void DownloadFromDCWidget::on_device_currentTextChanged(const QString &device)
+{
+#if defined(Q_OS_MACOS)
+	if (isBluetoothAddress(device)) {
+		// ensure we have a discovery running
+		if (btd == nullptr)
+			btd = BTDiscovery::instance();
+		btd->discoverAddress(device);
+	}
+#else
+	Q_UNUSED(device)
+#endif
+}
+
 void DownloadFromDCWidget::on_search_clicked()
 {
 	if (ui.vendor->currentText() == "Uemis" || ui.vendor->currentText() == "Garmin") {
@@ -338,13 +363,15 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 #if defined(BT_SUPPORT)
 	data->setBluetoothMode(ui.bluetoothMode->isChecked());
 	if (data->bluetoothMode()) {
-		// Get the selected device address from dialog or from preferences
+		// Get the selected device address
 		if (btDeviceSelectionDialog != NULL) {
 			data->setDevName(btDeviceSelectionDialog->getSelectedDeviceAddress());
 			data->setDevBluetoothName(btDeviceSelectionDialog->getSelectedDeviceName());
 		} else {
-			data->setDevName(qPrefDiveComputer::device());
-			data->setDevBluetoothName(qPrefDiveComputer::device_name());
+			QString name, address;
+			address = extractBluetoothNameAddress(ui.device->currentText(), name);
+			data->setDevName(address);
+			data->setDevBluetoothName(name);
 		}
 	} else
 		// this breaks an "else if" across lines... not happy...
@@ -371,10 +398,6 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 	qPrefDiveComputer::set_product(data->product());
 	qPrefDiveComputer::set_device(data->devName());
 
-#if defined(BT_SUPPORT)
-	qPrefDiveComputer::set_download_mode(ui.bluetoothMode->isChecked() ? DC_TRANSPORT_BLUETOOTH : DC_TRANSPORT_SERIAL);
-#endif
-
 	// before we start, remember where the dive_table ended
 	previousLast = dive_table.nr;
 	thread.start();
@@ -487,38 +510,35 @@ void DownloadFromDCWidget::on_cancel_clicked()
 
 void DownloadFromDCWidget::on_ok_clicked()
 {
-	struct dive *dive;
-
 	if (currentState != DONE && currentState != ERROR)
 		return;
 
-	// record all the dives in the 'real' dive_table
-	for (int i = 0; i < downloadTable.nr; i++) {
+	// delete non-selected dives
+	int total = downloadTable.nr;
+	int j = 0;
+	for (int i = 0; i < total; i++) {
 		if (diveImportedModel->data(diveImportedModel->index(i, 0), Qt::CheckStateRole) == Qt::Checked)
-			record_dive(downloadTable.dives[i]);
+			j++;
 		else
-			clear_dive(downloadTable.dives[i]);
-		downloadTable.dives[i] = NULL;
+			delete_dive_from_table(&downloadTable, j);
 	}
-	downloadTable.nr = 0;
 
-	int uniqId, idx;
-	// remember the last downloaded dive (on most dive computers this will be the chronologically
-	// first new dive) and select it again after processing all the dives
-	MainWindow::instance()->dive_list()->unselectDives();
-
-	dive = get_dive(dive_table.nr - 1);
-	if (dive != NULL) {
-		uniqId = get_dive(dive_table.nr - 1)->id;
-		process_imported_dives(preferDownloaded());
+	if (downloadTable.nr > 0) {
+		MainWindow::instance()->diveList->unselectDives();
+		// remember the last downloaded dive (on most dive computers this will be the chronologically
+		// first new dive) and select it again after processing all the dives
+		int uniqId = downloadTable.dives[downloadTable.nr - 1]->id;
+		process_imported_dives(&downloadTable, preferDownloaded(), true);
+		autogroup_dives();
+		Command::clear();
 		// after process_imported_dives does any merging or resorting needed, we need
 		// to recreate the model for the dive list so we can select the newest dive
 		MainWindow::instance()->recreateDiveList();
-		idx = get_idx_by_uniq_id(uniqId);
+		int idx = get_idx_by_uniq_id(uniqId);
 		// this shouldn't be necessary - but there are reports that somehow existing dives stay selected
 		// (but not visible as selected)
-		MainWindow::instance()->dive_list()->unselectDives();
-		MainWindow::instance()->dive_list()->selectDive(idx, true);
+		MainWindow::instance()->diveList->unselectDives();
+		MainWindow::instance()->diveList->selectDive(idx, true);
 	}
 
 	if (ostcFirmwareCheck && currentState == DONE) {
