@@ -6,7 +6,6 @@
 #include "core/qthelper.h"
 #include "desktop-widgets/mapwidget.h"
 #include "qt-models/filtermodels.h"
-#include "qt-models/divelocationmodel.h"
 #include "core/divesitehelpers.h"
 #include "desktop-widgets/modeldelegates.h"
 
@@ -18,8 +17,9 @@
 #include <QDesktopWidget>
 #include <QScrollBar>
 
-LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBox(parent), modified(false)
+LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBox(parent), modified(false), diveSite(nullptr)
 {
+	memset(&taxonomy, 0, sizeof(taxonomy));
 	ui.setupUi(this);
 	ui.diveSiteMessage->setCloseButtonVisible(false);
 
@@ -33,8 +33,6 @@ LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBo
 	ui.diveSiteMessage->addAction(acceptAction);
 	ui.diveSiteMessage->addAction(rejectAction);
 
-	connect(this, SIGNAL(startFilterDiveSite(uint32_t)), MultiFilterSortModel::instance(), SLOT(startFilterDiveSite(uint32_t)));
-	connect(this, SIGNAL(stopFilterDiveSite()), MultiFilterSortModel::instance(), SLOT(stopFilterDiveSite()));
 	connect(ui.geoCodeButton, SIGNAL(clicked()), this, SLOT(reverseGeocode()));
 	connect(this, SIGNAL(nameChanged(const QString &, const QString &)),
 		LocationFilterModel::instance(), SLOT(changeName(const QString &, const QString &)));
@@ -42,23 +40,12 @@ LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBo
 	connect(ui.diveSiteCoordinates, SIGNAL(returnPressed()), this, SLOT(updateLocationOnMap()));
 	ui.diveSiteCoordinates->installEventFilter(this);
 
-	SsrfSortFilterProxyModel *filter_model = new SsrfSortFilterProxyModel(this);
-	filter_model->setSourceModel(LocationInformationModel::instance());
-	filter_model->setFilterRow(filter_same_gps_cb);
-	ui.diveSiteListView->setModel(filter_model);
+	ui.diveSiteListView->setModel(&filter_model);
 	ui.diveSiteListView->setModelColumn(LocationInformationModel::NAME);
 	ui.diveSiteListView->installEventFilter(this);
 	// Map Management Code.
-	connect(this, &LocationInformationWidget::requestCoordinates,
-		MapWidget::instance(), &MapWidget::prepareForGetDiveCoordinates);
-	connect(this, &LocationInformationWidget::endRequestCoordinates,
-		MapWidget::instance(), &MapWidget::endGetDiveCoordinates);
 	connect(MapWidget::instance(), &MapWidget::coordinatesChanged,
 		this, &LocationInformationWidget::updateGpsCoordinates);
-	connect(this, &LocationInformationWidget::endEditDiveSite,
-		MapWidget::instance(), &MapWidget::repopulateLabels);
-	connect(this, &LocationInformationWidget::coordinatesChanged,
-		MapWidget::instance(), &MapWidget::updateCurrentDiveSiteCoordinatesToMap);
 }
 
 bool LocationInformationWidget::eventFilter(QObject *object, QEvent *ev)
@@ -70,7 +57,7 @@ bool LocationInformationWidget::eventFilter(QObject *object, QEvent *ev)
 		contextMenu.exec(ctx->globalPos());
 		return true;
 	} else if (ev->type() == QEvent::FocusOut && object == ui.diveSiteCoordinates) {
-		emit coordinatesChanged();
+		updateLocationOnMap();
 	}
 	return false;
 }
@@ -83,54 +70,60 @@ void LocationInformationWidget::enableLocationButtons(bool enable)
 
 void LocationInformationWidget::mergeSelectedDiveSites()
 {
+	if (!diveSite)
+		return;
 	if (QMessageBox::warning(MainWindow::instance(), tr("Merging dive sites"),
 				 tr("You are about to merge dive sites, you can't undo that action \n Are you sure you want to continue?"),
 				 QMessageBox::Ok, QMessageBox::Cancel) != QMessageBox::Ok)
 		return;
 
 	QModelIndexList selection = ui.diveSiteListView->selectionModel()->selectedIndexes();
-	uint32_t *selected_dive_sites = (uint32_t *)malloc(sizeof(uint32_t) * selection.count());
-	int i = 0;
+	// std::vector guarantees contiguous storage and can therefore be passed to C-code
+	std::vector<struct dive_site *> selected_dive_sites;
+	selected_dive_sites.reserve(selection.count());
 	Q_FOREACH (const QModelIndex &idx, selection) {
-		selected_dive_sites[i] = (uint32_t)idx.data(LocationInformationModel::UUID_ROLE).toInt();
-		i++;
+		dive_site *ds = idx.data(LocationInformationModel::DIVESITE_ROLE).value<dive_site *>();
+		if (ds)
+			selected_dive_sites.push_back(ds);
 	}
-	merge_dive_sites(displayed_dive_site.uuid, selected_dive_sites, i);
+	merge_dive_sites(diveSite, selected_dive_sites.data(), (int)selected_dive_sites.size());
 	LocationInformationModel::instance()->update();
 	QSortFilterProxyModel *m = (QSortFilterProxyModel *)ui.diveSiteListView->model();
 	m->invalidate();
-	free(selected_dive_sites);
 }
 
 void LocationInformationWidget::updateLabels()
 {
-	if (displayed_dive_site.name)
-		ui.diveSiteName->setText(displayed_dive_site.name);
+	if (!diveSite) {
+		clearLabels();
+		return;
+	}
+	if (diveSite->name)
+		ui.diveSiteName->setText(diveSite->name);
 	else
 		ui.diveSiteName->clear();
-	const char *country = taxonomy_get_country(&displayed_dive_site.taxonomy);
+	const char *country = taxonomy_get_country(&taxonomy);
 	if (country)
 		ui.diveSiteCountry->setText(country);
 	else
 		ui.diveSiteCountry->clear();
-	if (displayed_dive_site.description)
-		ui.diveSiteDescription->setText(displayed_dive_site.description);
+	if (diveSite->description)
+		ui.diveSiteDescription->setText(diveSite->description);
 	else
 		ui.diveSiteDescription->clear();
-	if (displayed_dive_site.notes)
-		ui.diveSiteNotes->setPlainText(displayed_dive_site.notes);
+	if (diveSite->notes)
+		ui.diveSiteNotes->setPlainText(diveSite->notes);
 	else
 		ui.diveSiteNotes->clear();
-	if (displayed_dive_site.latitude.udeg || displayed_dive_site.longitude.udeg) {
-		const char *coords = printGPSCoords(displayed_dive_site.latitude.udeg, displayed_dive_site.longitude.udeg);
+	if (has_location(&diveSite->location)) {
+		const char *coords = printGPSCoords(&diveSite->location);
 		ui.diveSiteCoordinates->setText(coords);
 		free((void *)coords);
 	} else {
 		ui.diveSiteCoordinates->clear();
 	}
 
-	ui.locationTags->setText(constructLocationTags(&displayed_dive_site, false));
-
+	ui.locationTags->setText(constructLocationTags(&taxonomy, false));
 }
 
 void LocationInformationWidget::clearLabels()
@@ -143,108 +136,106 @@ void LocationInformationWidget::clearLabels()
 	ui.locationTags->clear();
 }
 
-void LocationInformationWidget::updateGpsCoordinates()
+void LocationInformationWidget::updateGpsCoordinates(const location_t &location)
 {
 	QString oldText = ui.diveSiteCoordinates->text();
-	const char *coords = printGPSCoords(displayed_dive_site.latitude.udeg, displayed_dive_site.longitude.udeg);
+
+	const char *coords = printGPSCoords(&location);
 	ui.diveSiteCoordinates->setText(coords);
-	enableLocationButtons(dive_site_has_gps_location(&displayed_dive_site));
+	enableLocationButtons(has_location(&location));
 	free((void *)coords);
 	if (oldText != ui.diveSiteCoordinates->text())
 		markChangedWidget(ui.diveSiteCoordinates);
 }
 
+// Parse GPS text into latitude and longitude.
+// On error, false is returned and the output parameters are left unmodified.
+bool parseGpsText(const QString &text, location_t &location)
+{
+	double lat, lon;
+	if (parseGpsText(text, &lat, &lon)) {
+		location = create_location(lat, lon);
+		return true;
+	}
+	return false;
+}
+
 void LocationInformationWidget::acceptChanges()
 {
-	char *uiString;
-	struct dive_site *currentDs;
-	uiString = copy_qstring(ui.diveSiteName->text());
-	if (get_dive_site_by_uuid(displayed_dive_site.uuid) != NULL) {
-		currentDs = get_dive_site_by_uuid(displayed_dive_site.uuid);
-	} else {
+	if (!diveSite) {
 		qWarning() << "did not have valid dive site in LocationInformationWidget";
-		currentDs = get_dive_site_by_uuid(create_dive_site_from_current_dive(uiString));
-		displayed_dive.dive_site_uuid = currentDs->uuid;
+		return;
 	}
-	currentDs->latitude = displayed_dive_site.latitude;
-	currentDs->longitude = displayed_dive_site.longitude;
-	if (!same_string(uiString, currentDs->name)) {
-		emit nameChanged(QString(currentDs->name), ui.diveSiteName->text());
-		free(currentDs->name);
-		currentDs->name = uiString;
+
+	char *uiString;
+	uiString = copy_qstring(ui.diveSiteName->text());
+	if (!same_string(uiString, diveSite->name)) {
+		emit nameChanged(QString(diveSite->name), ui.diveSiteName->text());
+		free(diveSite->name);
+		diveSite->name = uiString;
 	} else {
 		free(uiString);
 	}
 	uiString = copy_qstring(ui.diveSiteDescription->text());
-	if (!same_string(uiString, currentDs->description)) {
-		free(currentDs->description);
-		currentDs->description = uiString;
+	if (!same_string(uiString, diveSite->description)) {
+		free(diveSite->description);
+		diveSite->description = uiString;
 	} else {
 		free(uiString);
 	}
 	uiString = copy_qstring(ui.diveSiteCountry->text());
-	// if the user entered a different contriy, first update the taxonomy
-	// for the displayed dive site; this below will get copied into the currentDs
-	if (!same_string(uiString, taxonomy_get_country(&displayed_dive_site.taxonomy)) &&
+	// if the user entered a different country, first update the local taxonomy
+	// this below will get copied into the diveSite
+	if (!same_string(uiString, taxonomy_get_country(&taxonomy)) &&
 	    !empty_string(uiString))
-		taxonomy_set_country(&displayed_dive_site.taxonomy, uiString, taxonomy_origin::GEOMANUAL);
+		taxonomy_set_country(&taxonomy, uiString, taxonomy_origin::GEOMANUAL);
 	else
 		free(uiString);
-	// now update the currentDs (which we then later copy back ontop of displayed_dive_site
-	copy_dive_site_taxonomy(&displayed_dive_site, currentDs);
+	// now update the diveSite
+	copy_taxonomy(&taxonomy, &diveSite->taxonomy);
 
 	uiString = copy_qstring(ui.diveSiteNotes->document()->toPlainText());
-	if (!same_string(uiString, currentDs->notes)) {
-		free(currentDs->notes);
-		currentDs->notes = uiString;
+	if (!same_string(uiString, diveSite->notes)) {
+		free(diveSite->notes);
+		diveSite->notes = uiString;
 	} else {
 		free(uiString);
 	}
 
-	if (!ui.diveSiteCoordinates->text().isEmpty()) {
-		double lat, lon;
-		if (parseGpsText(ui.diveSiteCoordinates->text(), &lat, &lon)) {
-			currentDs->latitude.udeg = lrint(lat * 1000000.0);
-			currentDs->longitude.udeg = lrint(lon * 1000000.0);
-		}
+	if (!ui.diveSiteCoordinates->text().isEmpty())
+		parseGpsText(ui.diveSiteCoordinates->text(), diveSite->location);
+	if (dive_site_is_empty(diveSite)) {
+		LocationInformationModel::instance()->removeRow(get_divesite_idx(diveSite));
+		displayed_dive.dive_site = nullptr;
+		diveSite = nullptr;
 	}
-	if (dive_site_is_empty(currentDs)) {
-		LocationInformationModel::instance()->removeRow(get_divesite_idx(currentDs));
-		displayed_dive.dive_site_uuid = 0;
-	}
-	copy_dive_site(currentDs, &displayed_dive_site);
 	mark_divelist_changed(true);
 	resetState();
-	emit endRequestCoordinates();
-	emit endEditDiveSite();
-	emit stopFilterDiveSite();
-	emit coordinatesChanged();
 }
 
 void LocationInformationWidget::rejectChanges()
 {
 	resetState();
-	emit endRequestCoordinates();
-	emit stopFilterDiveSite();
-	emit endEditDiveSite();
-	emit coordinatesChanged();
 }
 
-void LocationInformationWidget::showEvent(QShowEvent *ev)
+void LocationInformationWidget::initFields(dive_site *ds)
 {
-	if (displayed_dive_site.uuid) {
+	diveSite = ds;
+	if (ds) {
+		copy_taxonomy(&ds->taxonomy, &taxonomy);
+		filter_model.set(ds, ds->location);
 		updateLabels();
-		enableLocationButtons(dive_site_has_gps_location(&displayed_dive_site));
+		enableLocationButtons(dive_site_has_gps_location(ds));
 		QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(ui.diveSiteListView->model());
-		emit startFilterDiveSite(displayed_dive_site.uuid);
+		MultiFilterSortModel::instance()->startFilterDiveSite(ds);
 		if (m)
 			m->invalidate();
 	} else {
+		free_taxonomy(&taxonomy);
+		filter_model.set(0, location_t { degrees_t{ 0 }, degrees_t{ 0 } });
 		clearLabels();
 	}
-	emit requestCoordinates();
-
-	QGroupBox::showEvent(ev);
+	MapWidget::instance()->prepareForGetDiveCoordinates(ds);
 }
 
 void LocationInformationWidget::markChangedWidget(QWidget *w)
@@ -263,58 +254,62 @@ void LocationInformationWidget::resetState()
 {
 	modified = false;
 	resetPallete();
-	MainWindow::instance()->dive_list()->setEnabled(true);
+	MainWindow::instance()->diveList->setEnabled(true);
 	MainWindow::instance()->setEnabledToolbar(true);
 	ui.diveSiteMessage->setText(tr("Dive site management"));
+	MapWidget::instance()->endGetDiveCoordinates();
+	MapWidget::instance()->repopulateLabels();
+	MultiFilterSortModel::instance()->stopFilterDiveSite();
+	emit endEditDiveSite();
+	updateLocationOnMap();
 }
 
 void LocationInformationWidget::enableEdition()
 {
-	MainWindow::instance()->dive_list()->setEnabled(false);
+	MainWindow::instance()->diveList->setEnabled(false);
 	MainWindow::instance()->setEnabledToolbar(false);
 	ui.diveSiteMessage->setText(tr("You are editing a dive site"));
 }
 
 void LocationInformationWidget::on_diveSiteCoordinates_textChanged(const QString &text)
 {
-	uint lat = displayed_dive_site.latitude.udeg;
-	uint lon = displayed_dive_site.longitude.udeg;
-	const char *coords = printGPSCoords(lat, lon);
-	if (!same_string(qPrintable(text), coords)) {
-		double latitude, longitude;
-		if (parseGpsText(text, &latitude, &longitude)) {
-			displayed_dive_site.latitude.udeg = lrint(latitude * 1000000);
-			displayed_dive_site.longitude.udeg = lrint(longitude * 1000000);
+	if (!diveSite)
+		return;
+	location_t location;
+	bool ok_old = has_location(&diveSite->location);
+	bool ok = parseGpsText(text, location);
+	if (ok != ok_old || !same_location(&location, &diveSite->location)) {
+		if (ok) {
 			markChangedWidget(ui.diveSiteCoordinates);
-			enableLocationButtons(latitude != 0 && longitude != 0);
+			enableLocationButtons(true);
+			filter_model.setCoordinates(location);
 		} else {
 			enableLocationButtons(false);
 		}
 	}
-	free((void *)coords);
 }
 
 void LocationInformationWidget::on_diveSiteCountry_textChanged(const QString& text)
 {
-	if (!same_string(qPrintable(text), taxonomy_get_country(&displayed_dive_site.taxonomy)))
+	if (!same_string(qPrintable(text), taxonomy_get_country(&taxonomy)))
 		markChangedWidget(ui.diveSiteCountry);
 }
 
 void LocationInformationWidget::on_diveSiteDescription_textChanged(const QString &text)
 {
-	if (!same_string(qPrintable(text), displayed_dive_site.description))
+	if (diveSite && !same_string(qPrintable(text), diveSite->description))
 		markChangedWidget(ui.diveSiteDescription);
 }
 
 void LocationInformationWidget::on_diveSiteName_textChanged(const QString &text)
 {
-	if (!same_string(qPrintable(text), displayed_dive_site.name))
+	if (diveSite && !same_string(qPrintable(text), diveSite->name))
 		markChangedWidget(ui.diveSiteName);
 }
 
 void LocationInformationWidget::on_diveSiteNotes_textChanged()
 {
-	if (!same_string(qPrintable(ui.diveSiteNotes->toPlainText()), displayed_dive_site.notes))
+	if (diveSite && !same_string(qPrintable(ui.diveSiteNotes->toPlainText()), diveSite->notes))
 		markChangedWidget(ui.diveSiteNotes);
 }
 
@@ -330,14 +325,22 @@ void LocationInformationWidget::resetPallete()
 
 void LocationInformationWidget::reverseGeocode()
 {
-	ReverseGeoLookupThread *geoLookup = ReverseGeoLookupThread::instance();
-	geoLookup->run();
-	updateLabels();
+	location_t location;
+	if (!parseGpsText(ui.diveSiteCoordinates->text(), location))
+		return;
+	reverseGeoLookup(location.lat, location.lon, &taxonomy);
+	ui.locationTags->setText(constructLocationTags(&taxonomy, false));
 }
 
 void LocationInformationWidget::updateLocationOnMap()
 {
-	emit coordinatesChanged();
+	if (!diveSite)
+		return;
+	location_t location;
+	if (!parseGpsText(ui.diveSiteCoordinates->text(), location))
+		return;
+	MapWidget::instance()->updateDiveSiteCoordinates(diveSite, location);
+	filter_model.setCoordinates(location);
 }
 
 DiveLocationFilterProxyModel::DiveLocationFilterProxyModel(QObject*)
@@ -351,7 +354,7 @@ bool DiveLocationFilterProxyModel::filterAcceptsRow(int source_row, const QModel
 	if (source_row == 0)
 		return true;
 
-	QString sourceString = sourceModel()->index(source_row, DiveLocationModel::NAME).data(Qt::DisplayRole).toString();
+	QString sourceString = sourceModel()->index(source_row, LocationInformationModel::NAME).data(Qt::DisplayRole).toString();
 	return sourceString.toLower().contains(location_line_edit->text().toLower());
 }
 
@@ -360,7 +363,6 @@ bool DiveLocationFilterProxyModel::lessThan(const QModelIndex &source_left, cons
 	return source_left.data().toString() < source_right.data().toString();
 }
 
-
 DiveLocationModel::DiveLocationModel(QObject*)
 {
 	resetModel();
@@ -378,14 +380,13 @@ QVariant DiveLocationModel::data(const QModelIndex &index, int role) const
 	static const QIcon geoCode(":geotag-icon");
 
 	if (index.row() <= 1) { // two special cases.
-		if (index.column() == UUID) {
-			return RECENTLY_ADDED_DIVESITE;
-		}
+		if (index.column() == LocationInformationModel::DIVESITE)
+			return QVariant::fromValue<dive_site *>(RECENTLY_ADDED_DIVESITE);
 		switch (role) {
 		case Qt::DisplayRole:
 			return new_ds_value[index.row()];
 		case Qt::ToolTipRole:
-			return displayed_dive_site.uuid ?
+			return displayed_dive.dive_site ?
 				tr("Create a new dive site, copying relevant information from the current dive.") :
 				tr("Create a new dive site with this name");
 		case Qt::DecorationRole:
@@ -395,35 +396,12 @@ QVariant DiveLocationModel::data(const QModelIndex &index, int role) const
 
 	// The dive sites are -2 because of the first two items.
 	struct dive_site *ds = get_dive_site(index.row() - 2);
-	switch (role) {
-	case Qt::EditRole:
-	case Qt::DisplayRole:
-		switch (index.column()) {
-		case UUID:
-			return ds->uuid;
-		case NAME:
-			return ds->name;
-		case LATITUDE:
-			return ds->latitude.udeg;
-		case LONGITUDE:
-			return ds->longitude.udeg;
-		case DESCRIPTION:
-			return ds->description;
-		case NOTES:
-			return ds->name;
-		}
-		break;
-	case Qt::DecorationRole: {
-		if (dive_site_has_gps_location(ds))
-			return geoCode;
-	}
-	}
-	return QVariant();
+	return LocationInformationModel::getDiveSiteData(ds, index.column(), role);
 }
 
 int DiveLocationModel::columnCount(const QModelIndex&) const
 {
-	return COLUMNS;
+	return LocationInformationModel::COLUMNS;
 }
 
 int DiveLocationModel::rowCount(const QModelIndex&) const
@@ -448,16 +426,16 @@ DiveLocationLineEdit::DiveLocationLineEdit(QWidget *parent) : QLineEdit(parent),
 							      proxy(new DiveLocationFilterProxyModel()),
 							      model(new DiveLocationModel()),
 							      view(new DiveLocationListView()),
-							      currType(NO_DIVE_SITE)
+							      currType(NO_DIVE_SITE),
+							      currDs(nullptr)
 {
-	currUuid = 0;
 	location_line_edit = this;
 
 	proxy->setSourceModel(model);
-	proxy->setFilterKeyColumn(DiveLocationModel::NAME);
+	proxy->setFilterKeyColumn(LocationInformationModel::NAME);
 
 	view->setModel(proxy);
-	view->setModelColumn(DiveLocationModel::NAME);
+	view->setModelColumn(LocationInformationModel::NAME);
 	view->setItemDelegate(new LocationFilterDelegate());
 	view->setEditTriggers(QAbstractItemView::NoEditTriggers);
 	view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -526,21 +504,20 @@ void DiveLocationLineEdit::focusOutEvent(QFocusEvent *ev)
 void DiveLocationLineEdit::itemActivated(const QModelIndex &index)
 {
 	QModelIndex idx = index;
-	if (index.column() == DiveLocationModel::UUID)
-		idx = index.model()->index(index.row(), DiveLocationModel::NAME);
+	if (index.column() == LocationInformationModel::DIVESITE)
+		idx = index.model()->index(index.row(), LocationInformationModel::NAME);
 
-	QModelIndex uuidIndex = index.model()->index(index.row(), DiveLocationModel::UUID);
-	uint32_t uuid = uuidIndex.data().toInt();
-	currType = uuid == 1 ? NEW_DIVE_SITE : EXISTING_DIVE_SITE;
-	currUuid = uuid;
+	dive_site *ds = index.model()->index(index.row(), LocationInformationModel::DIVESITE).data().value<dive_site *>();
+	currType = ds == RECENTLY_ADDED_DIVESITE ? NEW_DIVE_SITE : EXISTING_DIVE_SITE;
+	currDs = ds;
 	setText(idx.data().toString());
-	if (currUuid == NEW_DIVE_SITE)
+	if (currType == NEW_DIVE_SITE)
 		qDebug() << "Setting a New dive site";
 	else
 		qDebug() << "Setting a Existing dive site";
 	if (view->isVisible())
 		view->hide();
-	emit diveSiteSelected(currUuid);
+	emit diveSiteSelected();
 }
 
 void DiveLocationLineEdit::refreshDiveSiteCache()
@@ -563,11 +540,18 @@ static struct dive_site *get_dive_site_name_start_which_str(const QString &str)
 
 void DiveLocationLineEdit::setTemporaryDiveSiteName(const QString&)
 {
-	QModelIndex i0 = model->index(0, DiveLocationModel::NAME);
-	QModelIndex i1 = model->index(1, DiveLocationModel::NAME);
+	// This function fills the first two entries with potential names of
+	// a dive site to be generated. The first entry is simply the entered
+	// text. The second entry is the first known dive site name starting
+	// with the entered text.
+	QModelIndex i0 = model->index(0, LocationInformationModel::NAME);
+	QModelIndex i1 = model->index(1, LocationInformationModel::NAME);
 	model->setData(i0, text());
 
-	QString i1_name = INVALID_DIVE_SITE_NAME;
+	// Note: if i1_name stays empty, the line will automatically
+	// be filtered out by the proxy filter, as it does not contain
+	// the user entered text.
+	QString i1_name;
 	if (struct dive_site *ds = get_dive_site_name_start_which_str(text())) {
 		const QString orig_name = QString(ds->name).toLower();
 		const QString new_name = text().toLower();
@@ -592,7 +576,7 @@ void DiveLocationLineEdit::keyPressEvent(QKeyEvent *ev)
 
 		if (ev->key() != Qt::Key_Up && ev->key() != Qt::Key_Down) {
 			currType = NEW_DIVE_SITE;
-			currUuid = RECENTLY_ADDED_DIVESITE;
+			currDs = RECENTLY_ADDED_DIVESITE;
 		} else {
 			showPopup();
 		}
@@ -638,17 +622,15 @@ void DiveLocationLineEdit::fixPopupPosition()
 	}
 }
 
-void DiveLocationLineEdit::setCurrentDiveSiteUuid(uint32_t uuid)
+void DiveLocationLineEdit::setCurrentDiveSite(struct dive_site *ds)
 {
-	currUuid = uuid;
-	if (uuid == 0) {
+	currDs = ds;
+	if (!currDs) {
 		currType = NO_DIVE_SITE;
-	}
-	struct dive_site *ds = get_dive_site_by_uuid(uuid);
-	if (!ds)
 		clear();
-	else
+	} else {
 		setText(ds->name);
+	}
 }
 
 void DiveLocationLineEdit::showPopup()
@@ -666,9 +648,9 @@ DiveLocationLineEdit::DiveSiteType DiveLocationLineEdit::currDiveSiteType() cons
 	return currType;
 }
 
-uint32_t DiveLocationLineEdit::currDiveSiteUuid() const
+struct dive_site *DiveLocationLineEdit::currDiveSite() const
 {
-	return currUuid;
+	return currDs;
 }
 
 DiveLocationListView::DiveLocationListView(QWidget*)
