@@ -34,7 +34,6 @@ class DivePlannerWidget;
 class ProfileWidget2;
 class PlannerDetails;
 class PlannerSettingsWidget;
-class QUndoStack;
 class LocationInformationWidget;
 
 typedef std::pair<QByteArray, QVariant> WidgetProperty;
@@ -60,36 +59,35 @@ public:
 	MainWindow();
 	~MainWindow();
 	static MainWindow *instance();
-	MainTab *information();
 	void loadRecentFiles();
 	void updateRecentFiles();
 	void updateRecentFilesMenu();
 	void addRecentFile(const QString &file, bool update);
-	DiveListView *dive_list();
-	DivePlannerWidget *divePlannerWidget();
-	PlannerSettingsWidget *divePlannerSettingsWidget();
 	LocationInformationWidget *locationInformationWidget();
 	void setTitle();
 
 	void loadFiles(const QStringList files);
 	void importFiles(const QStringList importFiles);
-	void importTxtFiles(const QStringList fileNames);
 	void cleanUpEmpty();
 	void setToolButtonsEnabled(bool enabled);
-	ProfileWidget2 *graphics() const;
-	PlannerDetails *plannerDetails() const;
 	void printPlan();
 	void checkSurvey();
 	void setApplicationState(const QByteArray& state);
 	void setStateProperties(const QByteArray& state, const PropertyList& tl, const PropertyList& tr, const PropertyList& bl,const PropertyList& br);
 	bool inPlanner();
-	QUndoStack *undoStack;
 	NotificationWidget *getNotificationWidget();
 	void enableDisableCloudActions();
+	void enableDisableOtherDCsActions();
 	void setCheckedActionFilterTags(bool checked);
 	void enterEditState();
 	void exitEditState();
 
+	MainTab *mainTab;
+	PlannerDetails *plannerDetails;
+	PlannerSettingsWidget *divePlannerSettingsWidget;
+	ProfileWidget2 *graphics;
+	DivePlannerWidget *divePlannerWidget;
+	DiveListView *diveList;
 private
 slots:
 	/* file menu action */
@@ -135,7 +133,7 @@ slots:
 	void on_action_Check_for_Updates_triggered();
 
 	void on_actionDiveSiteEdit_triggered();
-	void current_dive_changed();
+	void selectionChanged();
 	void initialUiSetup();
 
 	void on_actionImportDiveLog_triggered();
@@ -188,6 +186,8 @@ private:
 	Ui::MainWindow ui;
 	QAction *actionNextDive;
 	QAction *actionPreviousDive;
+	QAction *undoAction;
+	QAction *redoAction;
 #ifndef NO_USERMANUAL
 	UserManual *helpView;
 #endif
@@ -215,6 +215,7 @@ private:
 	void enterState(CurrentState);
 	bool filesAsArguments;
 	UpdateManager *updateManager;
+	LocationInformationWidget *diveSiteEdit;
 
 	bool plannerStateClean();
 	void setupForAddAndPlan(const char *model);
