@@ -11,6 +11,7 @@
 #include "qt-models/weightsysteminfomodel.h"
 #include "qt-models/weightmodel.h"
 #include "qt-models/divetripmodel.h"
+#include "qt-models/divelocationmodel.h"
 #include "core/qthelper.h"
 
 #include <QCompleter>
@@ -263,10 +264,10 @@ TankInfoDelegate::TankInfoDelegate(QObject *parent) : ComboBoxDelegate(TankInfoM
 
 void TankInfoDelegate::reenableReplot(QWidget*, QAbstractItemDelegate::EndEditHint)
 {
-	MainWindow::instance()->graphics()->setReplot(true);
+	MainWindow::instance()->graphics->setReplot(true);
 	// FIXME: We need to replot after a cylinder is selected but the replot below overwrites
 	//        the newly selected cylinder.
-	//	MainWindow::instance()->graphics()->replot();
+	//	MainWindow::instance()->graphics->replot();
 }
 
 void TankInfoDelegate::revertModelData(QWidget*, QAbstractItemDelegate::EndEditHint hint)
@@ -290,7 +291,7 @@ QWidget *TankInfoDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 	currCylinderData.type = copy_string(cyl->type.description);
 	currCylinderData.pressure = cyl->type.workingpressure.mbar;
 	currCylinderData.size = cyl->type.size.mliter;
-	MainWindow::instance()->graphics()->setReplot(false);
+	MainWindow::instance()->graphics->setReplot(false);
 	return delegate;
 }
 
@@ -452,9 +453,10 @@ void LocationFilterDelegate::paint(QPainter *painter, const QStyleOptionViewItem
 	QString diveSiteName = index.data().toString();
 	QString bottomText;
 	QIcon icon = index.data(Qt::DecorationRole).value<QIcon>();
-	struct dive_site *ds = get_dive_site_by_uuid(
-		index.model()->data(index.model()->index(index.row(),0)).toInt()
-	);
+	struct dive_site *ds =
+		index.model()->data(index.model()->index(index.row(), LocationInformationModel::DIVESITE)).value<dive_site *>();
+	struct dive_site *currentDiveSite = current_dive ? get_dive_site_for_dive(current_dive) : nullptr;
+	bool currentDiveSiteHasGPS = currentDiveSite && dive_site_has_gps_location(currentDiveSite);
 	//Special case: do not show name, but instead, show
 	if (index.row() < 2) {
 		diveSiteName = index.data().toString();
@@ -477,27 +479,26 @@ void LocationFilterDelegate::paint(QPainter *painter, const QStyleOptionViewItem
 	}
 
 	if (bottomText.isEmpty()) {
-		const char *gpsCoords = printGPSCoords(ds->latitude.udeg, ds->longitude.udeg);
+		const char *gpsCoords = printGPSCoords(&ds->location);
 		bottomText = QString(gpsCoords);
 		free( (void*) gpsCoords);
 	}
 
-	if (dive_site_has_gps_location(ds) && dive_site_has_gps_location(&displayed_dive_site)) {
+	if (dive_site_has_gps_location(ds) && currentDiveSiteHasGPS) {
 		// so we are showing a completion and both the current dive site and the completion
 		// have a GPS fix... so let's show the distance
-		if (ds->latitude.udeg == displayed_dive_site.latitude.udeg &&
-		    ds->longitude.udeg == displayed_dive_site.longitude.udeg) {
+		if (same_location(&ds->location, &currentDiveSite->location)) {
 			bottomText += tr(" (same GPS fix)");
 		} else {
-			int distanceMeters = get_distance(ds->latitude, ds->longitude, displayed_dive_site.latitude, displayed_dive_site.longitude);
+			int distanceMeters = get_distance(&ds->location, &currentDiveSite->location);
 			QString distance = distance_string(distanceMeters);
-			int nr = nr_of_dives_at_dive_site(ds->uuid, false);
+			int nr = nr_of_dives_at_dive_site(ds, false);
 			bottomText += tr(" (~%1 away").arg(distance);
 			bottomText += tr(", %n dive(s) here)", "", nr);
 		}
 	}
 	if (bottomText.isEmpty()) {
-		if (dive_site_has_gps_location(&displayed_dive_site))
+		if (currentDiveSiteHasGPS)
 			bottomText = tr("(no existing GPS data, add GPS fix from this dive)");
 		else
 			bottomText = tr("(no GPS data)");
