@@ -31,9 +31,11 @@ void PreferencesGraph::refreshSettings()
 	if (prefs.display_deco_mode == BUEHLMANN) {
 		ui->buehlmann->setChecked(true);
 		ui->vpmb->setChecked(false);
+		on_buehlmann_toggled(true);
 	} else {
 		ui->buehlmann->setChecked(false);
 		ui->vpmb->setChecked(true);
+		on_buehlmann_toggled(false);
 	}
 
 	ui->gflow->setValue(prefs.gflow);
@@ -69,7 +71,9 @@ void PreferencesGraph::syncSettings()
 	prefs.planner_deco_mode = ui->buehlmann->isChecked() ? BUEHLMANN : VPMB;
 	qPrefTechnicalDetails::set_gflow(ui->gflow->value());
 	qPrefTechnicalDetails::set_gfhigh(ui->gfhigh->value());
+	set_gf(ui->gflow->value(), ui->gfhigh->value());
 	qPrefTechnicalDetails::set_vpmb_conservatism(ui->vpmb_conservatism->value());
+	set_vpmb_conservatism(ui->vpmb_conservatism->value());
 	qPrefTechnicalDetails::set_show_ccr_setpoint(ui->show_ccr_setpoint->isChecked());
 	qPrefTechnicalDetails::set_show_ccr_sensors(ui->show_ccr_sensors->isChecked());
 	qPrefTechnicalDetails::set_show_scr_ocpo2(ui->show_scr_ocpo2->isChecked());
