@@ -45,7 +45,7 @@ static void copy_gps_location(struct dive *from, struct dive *to)
 		struct dive_site *gds = get_dive_site_for_dive(from);
 		if (!ds) {
 			// simply link to the one created for the fake dive
-			to->dive_site_uuid = gds->uuid;
+			to->dive_site = gds;
 		} else {
 			ds->latitude = gds->latitude;
 			ds->longitude = gds->longitude;
@@ -198,16 +198,13 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		/* make sure the buffer is empty and add the dive */
 		mb.len = 0;
 
-		struct dive_site *ds = get_dive_site_by_uuid(dive->dive_site_uuid);
+		struct dive_site *ds = dive->dive_site;
 
 		if (ds) {
-			put_format(&mb, "<divelog><divesites><site uuid='%8x' name='", dive->dive_site_uuid);
+			put_format(&mb, "<divelog><divesites><site uuid='%8x' name='", dive->dive_site->uuid);
 			put_quoted(&mb, ds->name, 1, 0);
 			put_format(&mb, "'");
-			if (ds->latitude.udeg || ds->longitude.udeg) {
-				put_degrees(&mb, ds->latitude, " gps='", " ");
-				put_degrees(&mb, ds->longitude, "", "'");
-			}
+			put_location(&mb, &ds->location, " gps='", "'");
 			put_format(&mb, ">\n");
 			if (ds->taxonomy.nr) {
 				for (int j = 0; j < ds->taxonomy.nr; j++) {
@@ -767,8 +764,9 @@ void DivelogsDeWebServices::buttonClicked(QAbstractButton *button)
 			break;
 		}
 		/* parse file and import dives */
-		parse_file(QFile::encodeName(zipFile.fileName()), &dive_table);
-		process_imported_dives(false);
+		struct dive_table table = { 0 };
+		parse_file(QFile::encodeName(zipFile.fileName()), &table);
+		process_imported_dives(&table, false, false);
 		MainWindow::instance()->refreshDisplay();
 
 		/* store last entered user/pass in config */
@@ -793,17 +791,16 @@ void DivelogsDeWebServices::buttonClicked(QAbstractButton *button)
 	}
 }
 
-UserSurveyServices::UserSurveyServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f)
+UserSurveyServices::UserSurveyServices(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f)
 {
-
 }
 
-QNetworkReply* UserSurveyServices::sendSurvey(QString values)
+QNetworkReply *UserSurveyServices::sendSurvey(QString values)
 {
 	QNetworkRequest request;
 	request.setUrl(QString("http://subsurface-divelog.org/survey?%1").arg(values));
 	request.setRawHeader("Accept", "text/xml");
-	request.setRawHeader("User-Agent", userAgent.toUtf8());
-	reply = manager()->get(request);
+	request.setRawHeader("User-Agent", getUserAgent().toUtf8());
+	QNetworkReply *reply = manager()->get(request);
 	return reply;
 }
