@@ -26,6 +26,7 @@
 #include "core/subsurface-string.h"
 #include "core/gettextfromc.h"
 #include "desktop-widgets/locationinformation.h"
+#include "desktop-widgets/command.h"
 
 #include "TabDiveExtraInfo.h"
 #include "TabDiveInformation.h"
@@ -194,21 +195,6 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 		ui.cylinders->view()->horizontalHeader()->addAction(action);
 	}
 
-	ui.waitingSpinner->setRoundness(70.0);
-	ui.waitingSpinner->setMinimumTrailOpacity(15.0);
-	ui.waitingSpinner->setTrailFadePercentage(70.0);
-	ui.waitingSpinner->setNumberOfLines(8);
-	ui.waitingSpinner->setLineLength(5);
-	ui.waitingSpinner->setLineWidth(3);
-	ui.waitingSpinner->setInnerRadius(5);
-	ui.waitingSpinner->setRevolutionsPerSecond(1);
-
-	connect(ReverseGeoLookupThread::instance(), SIGNAL(finished()),
-			LocationInformationModel::instance(), SLOT(update()));
-
-	connect(ReverseGeoLookupThread::instance(), &QThread::finished,
-			this, &MainTab::setCurrentLocationIndex);
-
 	connect(ui.diveNotesMessage, &KMessageWidget::showAnimationFinished,
 					ui.location, &DiveLocationLineEdit::fixPopupPosition);
 
@@ -236,24 +222,14 @@ MainTab::~MainTab()
 void MainTab::setCurrentLocationIndex()
 {
 	if (current_dive) {
-		struct dive_site *ds = get_dive_site_by_uuid(current_dive->dive_site_uuid);
+		struct dive_site *ds = current_dive->dive_site;
 		if (ds)
-			ui.location->setCurrentDiveSiteUuid(ds->uuid);
+			ui.location->setCurrentDiveSite(ds);
 		else
 			ui.location->clear();
 	}
 }
 
-void MainTab::enableGeoLookupEdition()
-{
-	ui.waitingSpinner->stop();
-}
-
-void MainTab::disableGeoLookupEdition()
-{
-	ui.waitingSpinner->start();
-}
-
 void MainTab::toggleTriggeredColumn()
 {
 	QAction *action = qobject_cast<QAction *>(sender());
@@ -317,7 +293,7 @@ void MainTab::updateTextLabels(bool showUnits)
 void MainTab::enableEdition(EditMode newEditMode)
 {
 	const bool isTripEdit = MainWindow::instance() &&
-		MainWindow::instance()->dive_list()->selectedTrips().count() == 1;
+		MainWindow::instance()->diveList->selectedTrips().count() == 1;
 
 	if (((newEditMode == DIVE || newEditMode == NONE) && current_dive == NULL) || editMode != NONE)
 		return;
@@ -341,7 +317,7 @@ void MainTab::enableEdition(EditMode newEditMode)
 	}
 
 	ui.editDiveSiteButton->setEnabled(false);
-	MainWindow::instance()->dive_list()->setEnabled(false);
+	MainWindow::instance()->diveList->setEnabled(false);
 	MainWindow::instance()->setEnabledToolbar(false);
 	MainWindow::instance()->enterEditState();
 	ui.tabWidget->setTabEnabled(2, false);
@@ -393,14 +369,6 @@ bool MainTab::isEditing()
 	return editMode != NONE;
 }
 
-void MainTab::showLocation()
-{
-	if (get_dive_site_by_uuid(displayed_dive.dive_site_uuid))
-		ui.location->setCurrentDiveSiteUuid(displayed_dive.dive_site_uuid);
-	else
-		ui.location->clear();
-}
-
 void MainTab::updateDepthDuration()
 {
 	ui.depth->setVisible(true);
@@ -416,7 +384,7 @@ void MainTab::updateDiveInfo(bool clear)
 	ui.location->refreshDiveSiteCache();
 	EditMode rememberEM = editMode;
 	// don't execute this while adding / planning a dive
-	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE || MainWindow::instance()->graphics()->isPlanner())
+	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE || MainWindow::instance()->graphics->isPlanner())
 		return;
 	if (!isEnabled() && !clear )
 		setEnabled(true);
@@ -424,16 +392,6 @@ void MainTab::updateDiveInfo(bool clear)
 		setEnabled(false);
 	editMode = IGNORE; // don't trigger on changes to the widgets
 
-	// This method updates ALL tabs whenever a new dive or trip is
-	// selected.
-	// If exactly one trip has been selected, we show the location / notes
-	// for the trip in the Info tab, otherwise we show the info of the
-	// selected_dive
-	struct dive *prevd;
-
-	process_selected_dives();
-	process_all_dives(&displayed_dive, &prevd);
-
 	for (auto widget : extraWidgets) {
 		widget->updateData();
 	}
@@ -457,24 +415,19 @@ void MainTab::updateDiveInfo(bool clear)
 
 	if (!clear) {
 		struct dive_site *ds = NULL;
-		// if we are showing a dive and editing it, let's refer to the displayed_dive_site as that
-		// already may contain changes, otherwise start with the dive site referred to by the displayed
-		// dive
-		if (rememberEM == DIVE) {
-			ds = &displayed_dive_site;
-		} else {
-			ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
-			if (ds)
-				copy_dive_site(ds, &displayed_dive_site);
-		}
-
+		ds = displayed_dive.dive_site;
 		if (ds) {
-			ui.location->setCurrentDiveSiteUuid(ds->uuid);
-			ui.locationTags->setText(constructLocationTags(ds, true));
+			ui.location->setCurrentDiveSite(ds);
+			ui.locationTags->setText(constructLocationTags(&ds->taxonomy, true));
+
+			if (ui.locationTags->text().isEmpty() && has_location(&ds->location)) {
+				const char *coords = printGPSCoords(&ds->location);
+				ui.locationTags->setText(coords);
+				free((void *)coords);
+			}
 		} else {
 			ui.location->clear();
 			ui.locationTags->clear();
-			clear_dive_site(&displayed_dive_site);
 		}
 
 		// Subsurface always uses "local time" as in "whatever was the local time at the location"
@@ -483,7 +436,7 @@ void MainTab::updateDiveInfo(bool clear)
 		localTime.setTimeSpec(Qt::UTC);
 		ui.dateEdit->setDate(localTime.date());
 		ui.timeEdit->setTime(localTime.time());
-		if (MainWindow::instance() && MainWindow::instance()->dive_list()->selectedTrips().count() == 1) {
+		if (MainWindow::instance() && MainWindow::instance()->diveList->selectedTrips().count() == 1) {
 			// Remember the tab selected for last dive
 			if (lastSelectedDive)
 				lastTabSelectedDive = ui.tabWidget->currentIndex();
@@ -495,7 +448,7 @@ void MainTab::updateDiveInfo(bool clear)
 			if (lastSelectedDive)
 				ui.tabWidget->setCurrentIndex(lastTabSelectedDiveTrip);
 			lastSelectedDive = false;
-			currentTrip = *MainWindow::instance()->dive_list()->selectedTrips().begin();
+			currentTrip = *MainWindow::instance()->diveList->selectedTrips().begin();
 			// only use trip relevant fields
 			ui.divemaster->setVisible(false);
 			ui.DivemasterLabel->setVisible(false);
@@ -651,9 +604,9 @@ void MainTab::updateDiveInfo(bool clear)
 	else
 		ui.cylinders->view()->hideColumn(CylindersModel::USE);
 
-	if (verbose)
-		qDebug() << "Set the current dive site:" << displayed_dive.dive_site_uuid;
-	emit diveSiteChanged(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
+	if (verbose && displayed_dive.dive_site)
+		qDebug() << "Set the current dive site:" << displayed_dive.dive_site->uuid;
+	emit diveSiteChanged();
 }
 
 void MainTab::addCylinder_clicked()
@@ -680,21 +633,12 @@ void MainTab::reload()
 }
 
 // tricky little macro to edit all the selected dives
-// loop over all dives, for each selected dive do WHAT, but do it
-// last for the current dive; this is required in case the invocation
-// wants to compare things to the original value in current_dive like it should
-#define MODIFY_SELECTED_DIVES(WHAT)                            \
+// loop ove all DIVES and do WHAT.
+#define MODIFY_DIVES(DIVES, WHAT)                            \
 	do {                                                 \
-		struct dive *mydive = NULL;                  \
-		int _i;                                      \
-		for_each_dive (_i, mydive) {                 \
-			if (!mydive->selected || mydive == cd) \
-				continue;                    \
-							     \
+		for (dive *mydive: DIVES) {                  \
 			WHAT;                                \
-		}                                            \
-		mydive = cd;                                 \
-		WHAT;                                        \
+		}					     \
 		mark_divelist_changed(true);                 \
 	} while (0)
 
@@ -716,70 +660,68 @@ MainTab::EditMode MainTab::getEditMode() const
 
 void MainTab::refreshDisplayedDiveSite()
 {
-	if (displayed_dive_site.uuid) {
-		copy_dive_site(get_dive_site_by_uuid(displayed_dive_site.uuid), &displayed_dive_site);
-		ui.location->setCurrentDiveSiteUuid(displayed_dive_site.uuid);
-	}
+	struct dive_site *ds = get_dive_site_for_dive(&displayed_dive);
+	if (ds)
+		ui.location->setCurrentDiveSite(ds);
 }
 
 // when this is called we already have updated the current_dive and know that it exists
 // there is no point in calling this function if there is no current dive
-uint32_t MainTab::updateDiveSite(uint32_t pickedUuid, int divenr)
+struct dive_site *MainTab::updateDiveSite(struct dive_site *pickedDs, dive *d)
 {
-	struct dive *cd = get_dive(divenr);
-	if (!cd)
+	if (!d)
 		return 0;
 
 	if (ui.location->text().isEmpty())
 		return 0;
 
-	if (pickedUuid == 0)
+	if (!pickedDs)
 		return 0;
 
-	const uint32_t origUuid = cd->dive_site_uuid;
-	struct dive_site *origDs = get_dive_site_by_uuid(origUuid);
-	struct dive_site *newDs = NULL;
+	struct dive_site *origDs = d->dive_site;
 	bool createdNewDive = false;
 
-	if (pickedUuid == origUuid)
-		return origUuid;
+	if (pickedDs == origDs)
+		return origDs;
 
-	if (pickedUuid == RECENTLY_ADDED_DIVESITE) {
+	if (pickedDs == RECENTLY_ADDED_DIVESITE) {
 		QString name = ui.location->text().isEmpty() ? tr("New dive site") : ui.location->text();
-		pickedUuid = create_dive_site(qPrintable(name), displayed_dive.when);
+		pickedDs = create_dive_site(qPrintable(name), displayed_dive.when);
 		createdNewDive = true;
 		LocationFilterModel::instance()->addName(name);
 	}
 
-	newDs = get_dive_site_by_uuid(pickedUuid);
-
-	// Copy everything from the displayed_dive_site, so we have the latitude, longitude, notes, etc.
-	// The user *might* be using wrongly the 'choose dive site' just to edit the name of it, sigh.
 	if (origDs) {
 		if(createdNewDive) {
-			copy_dive_site(origDs, newDs);
-			free(newDs->name);
-			newDs->name = copy_qstring(ui.location->text());
-			newDs->uuid = pickedUuid;
+			uint32_t pickedUuid = pickedDs->uuid;
+			copy_dive_site(origDs, pickedDs);
+			free(pickedDs->name);
+			pickedDs->name = copy_qstring(ui.location->text());
+			pickedDs->uuid = pickedUuid;
 			qDebug() << "Creating and copying dive site";
-		} else if (newDs->latitude.udeg == 0 && newDs->longitude.udeg == 0) {
-			newDs->latitude.udeg = origDs->latitude.udeg;
-			newDs->longitude.udeg = origDs->longitude.udeg;
+		} else if (!has_location(&pickedDs->location)) {
+			pickedDs->location = origDs->location;
 			qDebug() << "Copying GPS information";
 		}
 	}
 
-	if (origDs && pickedUuid != origDs->uuid && same_string(origDs->notes, "SubsurfaceWebservice")) {
-		if (!is_dive_site_used(origDs->uuid, false)) {
-			if (verbose)
-				qDebug() << "delete the autogenerated dive site" << origDs->name;
-			delete_dive_site(origDs->uuid);
-		}
-	}
+	d->dive_site = pickedDs;
+	qDebug() << "Setting the dive site id on the dive:" << pickedDs->uuid;
+	return pickedDs;
+}
 
-	cd->dive_site_uuid = pickedUuid;
-	qDebug() << "Setting the dive site id on the dive:" << pickedUuid;
-	return pickedUuid;
+// Get the list of selected dives, but put the current dive at the last position of the vector
+static QVector<dive *> getSelectedDivesCurrentLast()
+{
+	QVector<dive *> res;
+	struct dive *d;
+	int i;
+	for_each_dive (i, d) {
+		if (d->selected && d != current_dive)
+			res.append(d);
+	}
+	res.append(current_dive);
+	return res;
 }
 
 void MainTab::acceptChanges()
@@ -799,24 +741,26 @@ void MainTab::acceptChanges()
 	hideMessage();
 	ui.equipmentTab->setEnabled(true);
 	if (editMode == ADD) {
-		// We need to add the dive we just created to the dive list and select it.
-		// Easy, right?
-		struct dive *added_dive = clone_dive(&displayed_dive);
-		record_dive(added_dive);
-		addedId = added_dive->id;
 		// make sure that the dive site is handled as well
-		updateDiveSite(ui.location->currDiveSiteUuid(), get_idx_by_uniq_id(added_dive->id));
+		updateDiveSite(ui.location->currDiveSite(), &displayed_dive);
+		copyTagsToDisplayedDive();
 
-		// unselect everything as far as the UI is concerned and select the new
-		// dive - we'll have to undo/redo this later after we resort the dive_table
-		// but we need the dive selected for the middle part of this function - this
-		// way we can reuse the code used for editing dives
-		MainWindow::instance()->dive_list()->unselectDives();
-		selected_dive = get_divenr(added_dive);
-		amount_selected = 1;
-		// finally, make sure we get the tags
-		saveTags();
-	} else if (MainWindow::instance() && MainWindow::instance()->dive_list()->selectedTrips().count() == 1) {
+		Command::addDive(&displayed_dive, autogroup, true);
+
+		editMode = NONE;
+		MainWindow::instance()->exitEditState();
+		cylindersModel->changed = false;
+		weightModel->changed = false;
+		MainWindow::instance()->setEnabledToolbar(true);
+		acceptingEdit = false;
+		ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
+		emit addDiveFinished();
+		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
+		MainWindow::instance()->diveList->setFocus();
+		resetPallete();
+		displayed_dive.divetrip = nullptr; // Should not be necessary, just in case!
+		return;
+	} else if (MainWindow::instance() && MainWindow::instance()->diveList->selectedTrips().count() == 1) {
 		/* now figure out if things have changed */
 		if (displayedTrip.notes && !same_string(displayedTrip.notes, currentTrip->notes)) {
 			currentTrip->notes = copy_string(displayedTrip.notes);
@@ -829,6 +773,10 @@ void MainTab::acceptChanges()
 		currentTrip = NULL;
 		ui.dateEdit->setEnabled(true);
 	} else {
+		// Get list of selected dives, but put the current dive last;
+		// this is required in case the invocation wants to compare things
+		// to the original value in current_dive like it should
+		QVector<dive *> selectedDives = getSelectedDivesCurrentLast();
 		if (editMode == MANUALLY_ADDED_DIVE) {
 			// preserve any changes to the profile
 			free(current_dive->dc.sample);
@@ -839,42 +787,37 @@ void MainTab::acceptChanges()
 		// now check if something has changed and if yes, edit the selected dives that
 		// were identical with the master dive shown (and mark the divelist as changed)
 		if (!same_string(displayed_dive.suit, cd->suit))
-			MODIFY_SELECTED_DIVES(EDIT_TEXT(suit));
+			MODIFY_DIVES(selectedDives, EDIT_TEXT(suit));
 		if (!same_string(displayed_dive.notes, cd->notes))
-			MODIFY_SELECTED_DIVES(EDIT_TEXT(notes));
+			MODIFY_DIVES(selectedDives, EDIT_TEXT(notes));
 		if (displayed_dive.rating != cd->rating)
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(rating));
+			MODIFY_DIVES(selectedDives, EDIT_VALUE(rating));
 		if (displayed_dive.visibility != cd->visibility)
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(visibility));
+			MODIFY_DIVES(selectedDives, EDIT_VALUE(visibility));
 		if (displayed_dive.airtemp.mkelvin != cd->airtemp.mkelvin)
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(airtemp.mkelvin));
+			MODIFY_DIVES(selectedDives, EDIT_VALUE(airtemp.mkelvin));
 		if (displayed_dc->divemode != current_dc->divemode) {
-			MODIFY_SELECTED_DIVES(
-				if (get_dive_dc(mydive, dc_number)->divemode == current_dc->divemode || copyPaste) {
+			MODIFY_DIVES(selectedDives,
+				if (get_dive_dc(mydive, dc_number)->divemode == current_dc->divemode || copyPaste)
 					get_dive_dc(mydive, dc_number)->divemode = displayed_dc->divemode;
-				}
 			);
-			MODIFY_SELECTED_DIVES(update_setpoint_events(mydive, get_dive_dc(mydive, dc_number)));
+			MODIFY_DIVES(selectedDives, update_setpoint_events(mydive, get_dive_dc(mydive, dc_number)));
 			do_replot = true;
 		}
 		if (displayed_dive.watertemp.mkelvin != cd->watertemp.mkelvin)
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(watertemp.mkelvin));
-		if (displayed_dive.when != cd->when) {
-			time_t offset = cd->when - displayed_dive.when;
-			MODIFY_SELECTED_DIVES(mydive->when -= offset;);
-		}
+			MODIFY_DIVES(selectedDives, EDIT_VALUE(watertemp.mkelvin));
 
-		if (displayed_dive.dive_site_uuid != cd->dive_site_uuid)
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(dive_site_uuid));
+		if (displayed_dive.dive_site != cd->dive_site)
+			MODIFY_DIVES(selectedDives, EDIT_VALUE(dive_site));
 
 		// three text fields are somewhat special and are represented as tags
 		// in the UI - they need somewhat smarter handling
-		saveTaggedStrings();
-		saveTags();
+		saveTaggedStrings(selectedDives);
+		saveTags(selectedDives);
 
 		if (editMode != ADD && cylindersModel->changed) {
 			mark_divelist_changed(true);
-			MODIFY_SELECTED_DIVES(
+			MODIFY_DIVES(selectedDives,
 				for (int i = 0; i < MAX_CYLINDERS; i++) {
 					if (mydive != cd) {
 						if (same_string(mydive->cylinder[i].type.description, cd->cylinder[i].type.description) || copyPaste) {
@@ -918,7 +861,7 @@ void MainTab::acceptChanges()
 
 		if (weightModel->changed) {
 			mark_divelist_changed(true);
-			MODIFY_SELECTED_DIVES(
+			MODIFY_DIVES(selectedDives,
 				for (int i = 0; i < MAX_WEIGHTSYSTEMS; i++) {
 					if (mydive != cd && (copyPaste || same_string(mydive->weightsystem[i].description, cd->weightsystem[i].description))) {
 						mydive->weightsystem[i] = displayed_dive.weightsystem[i];
@@ -933,29 +876,23 @@ void MainTab::acceptChanges()
 		}
 
 		// update the dive site for the selected dives that had the same dive site as the current dive
-		uint32_t oldUuid = cd->dive_site_uuid;
-		uint32_t newUuid = 0;
-		MODIFY_SELECTED_DIVES(
-			if (mydive->dive_site_uuid == current_dive->dive_site_uuid) {
-				newUuid = updateDiveSite(newUuid == 0 ? ui.location->currDiveSiteUuid() : newUuid,  get_idx_by_uniq_id(mydive->id));
-			}
+		struct dive_site *oldDs = cd->dive_site;
+		struct dive_site *newDs = nullptr;
+		MODIFY_DIVES(selectedDives,
+			if (mydive->dive_site == current_dive->dive_site)
+				newDs = updateDiveSite(!newDs ? ui.location->currDiveSite() : newDs, mydive);
 		);
-		if (!is_dive_site_used(oldUuid, false)) {
-			if (verbose) {
-				struct dive_site *ds = get_dive_site_by_uuid(oldUuid);
-				qDebug() << "delete now unused dive site" << ((ds && ds->name) ? ds->name : "without name");
-			}
-			delete_dive_site(oldUuid);
+		if (oldDs && !is_dive_site_used(oldDs, false)) {
+			if (verbose)
+				qDebug() << "delete now unused dive site" << (oldDs->name ? oldDs->name : "without name");
+			delete_dive_site(oldDs);
 			MapWidget::instance()->reload();
 		}
 		// the code above can change the correct uuid for the displayed dive site - and the
 		// code below triggers an update of the display without re-initializing displayed_dive
 		// so let's make sure here that our data is consistent now that we have handled the
 		// dive sites
-		displayed_dive.dive_site_uuid = current_dive->dive_site_uuid;
-		struct dive_site *ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
-		if (ds)
-			copy_dive_site(ds, &displayed_dive_site);
+		displayed_dive.dive_site = current_dive->dive_site;
 
 		// each dive that was selected might have had the temperatures in its active divecomputer changed
 		// so re-populate the temperatures - easiest way to do this is by calling fixup_dive
@@ -965,12 +902,18 @@ void MainTab::acceptChanges()
 				invalidate_dive_cache(d);
 			}
 		}
+
+		if (displayed_dive.when != cd->when) {
+			timestamp_t offset = cd->when - displayed_dive.when;
+			if (offset)
+				Command::shiftTime(selectedDives, (int)offset);
+		}
 	}
 	if (editMode != TRIP && current_dive->divetrip) {
 		current_dive->divetrip->when = current_dive->when;
 		find_new_trip_start_time(current_dive->divetrip);
 	}
-	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE) {
+	if (editMode == MANUALLY_ADDED_DIVE) {
 		// we just added or edited the dive, let fixup_dive() make
 		// sure we get the max. depth right
 		current_dive->maxdepth.mm = current_dc->maxdepth.mm = 0;
@@ -980,32 +923,27 @@ void MainTab::acceptChanges()
 		mark_divelist_changed(true);
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
 	}
-	int scrolledBy = MainWindow::instance()->dive_list()->verticalScrollBar()->sliderPosition();
+	int scrolledBy = MainWindow::instance()->diveList->verticalScrollBar()->sliderPosition();
 	resetPallete();
-	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE) {
-		// since a newly added dive could be in the middle of the dive_table we need
-		// to resort the dive list and make sure the newly added dive gets selected again
-		sort_table(&dive_table);
-		MainWindow::instance()->dive_list()->reload(DiveTripModel::CURRENT, true);
+	if (editMode == MANUALLY_ADDED_DIVE) {
+		MainWindow::instance()->diveList->reload();
 		int newDiveNr = get_divenr(get_dive_by_uniq_id(addedId));
-		MainWindow::instance()->dive_list()->unselectDives();
-		MainWindow::instance()->dive_list()->selectDive(newDiveNr, true);
+		MainWindow::instance()->diveList->unselectDives();
+		MainWindow::instance()->diveList->selectDive(newDiveNr, true);
 		editMode = NONE;
 		MainWindow::instance()->refreshDisplay();
-		MainWindow::instance()->graphics()->replot();
-		emit addDiveFinished();
+		MainWindow::instance()->graphics->replot();
 	} else {
 		editMode = NONE;
 		if (do_replot)
-			MainWindow::instance()->graphics()->replot();
-		MainWindow::instance()->dive_list()->rememberSelection();
-		sort_table(&dive_table);
+			MainWindow::instance()->graphics->replot();
+		MainWindow::instance()->diveList->rememberSelection();
 		MainWindow::instance()->refreshDisplay();
-		MainWindow::instance()->dive_list()->restoreSelection();
+		MainWindow::instance()->diveList->restoreSelection();
 	}
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
-	MainWindow::instance()->dive_list()->verticalScrollBar()->setSliderPosition(scrolledBy);
-	MainWindow::instance()->dive_list()->setFocus();
+	MainWindow::instance()->diveList->verticalScrollBar()->setSliderPosition(scrolledBy);
+	MainWindow::instance()->diveList->setFocus();
 	MainWindow::instance()->exitEditState();
 	cylindersModel->changed = false;
 	weightModel->changed = false;
@@ -1056,23 +994,23 @@ void MainTab::rejectChanges()
 	// no harm done to call cancelPlan even if we were not in ADD or PLAN mode...
 	DivePlannerPointsModel::instance()->cancelPlan();
 	if(lastMode == ADD)
-		MainWindow::instance()->dive_list()->restoreSelection();
+		MainWindow::instance()->diveList->restoreSelection();
 
 	// now make sure that the correct dive is displayed
-	if (selected_dive >= 0)
+	if (current_dive)
 		copy_dive(current_dive, &displayed_dive);
 	else
 		clear_dive(&displayed_dive);
-	updateDiveInfo(selected_dive < 0);
+	updateDiveInfo(!current_dive);
 
 	for (auto widget : extraWidgets) {
 		widget->updateData();
 	}
 	// the user could have edited the location and then canceled the edit
 	// let's get the correct location back in view
-	MapWidget::instance()->centerOnDiveSite(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
+	MapWidget::instance()->centerOnDiveSite(displayed_dive.dive_site);
 	// show the profile and dive info
-	MainWindow::instance()->graphics()->replot();
+	MainWindow::instance()->graphics->replot();
 	MainWindow::instance()->setEnabledToolbar(true);
 	MainWindow::instance()->exitEditState();
 	cylindersModel->changed = false;
@@ -1132,7 +1070,7 @@ void MainTab::on_duration_textChanged(const QString &text)
 	if (editMode == IGNORE || acceptingEdit == true)
 		return;
 	// parse this
-	MainWindow::instance()->graphics()->setReplot(false);
+	MainWindow::instance()->graphics->setReplot(false);
 	if (!isEditing())
 		enableEdition();
 	displayed_dive.dc.duration.seconds = parseDurationToSeconds(text);
@@ -1141,8 +1079,8 @@ void MainTab::on_duration_textChanged(const QString &text)
 	displayed_dive.dc.samples = 0;
 	DivePlannerPointsModel::instance()->loadFromDive(&displayed_dive);
 	markChangedWidget(ui.duration);
-	MainWindow::instance()->graphics()->setReplot(true);
-	MainWindow::instance()->graphics()->plotDive();
+	MainWindow::instance()->graphics->setReplot(true);
+	MainWindow::instance()->graphics->plotDive();
 
 }
 
@@ -1151,7 +1089,7 @@ void MainTab::on_depth_textChanged(const QString &text)
 	if (editMode == IGNORE || acceptingEdit == true)
 		return;
 	// don't replot until we set things up the way we want them
-	MainWindow::instance()->graphics()->setReplot(false);
+	MainWindow::instance()->graphics->setReplot(false);
 	if (!isEditing())
 		enableEdition();
 	displayed_dive.dc.maxdepth.mm = parseLengthToMm(text);
@@ -1160,8 +1098,8 @@ void MainTab::on_depth_textChanged(const QString &text)
 	displayed_dive.dc.samples = 0;
 	DivePlannerPointsModel::instance()->loadFromDive(&displayed_dive);
 	markChangedWidget(ui.depth);
-	MainWindow::instance()->graphics()->setReplot(true);
-	MainWindow::instance()->graphics()->plotDive();
+	MainWindow::instance()->graphics->setReplot(true);
+	MainWindow::instance()->graphics->plotDive();
 }
 
 void MainTab::on_airtemp_textChanged(const QString &text)
@@ -1180,7 +1118,7 @@ void MainTab::divetype_Changed(int index)
 	displayed_dc->divemode = (enum divemode_t) index;
 	update_setpoint_events(&displayed_dive, displayed_dc);
 	markChangedWidget(ui.DiveType);
-	MainWindow::instance()->graphics()->recalcCeiling();
+	MainWindow::instance()->graphics->recalcCeiling();
 }
 
 void MainTab::on_watertemp_textChanged(const QString &text)
@@ -1244,25 +1182,31 @@ void MainTab::on_timeEdit_timeChanged(const QTime &time)
 	emit dateTimeChanged();
 }
 
+void MainTab::copyTagsToDisplayedDive()
+{
+	taglist_free(displayed_dive.tag_list);
+	displayed_dive.tag_list = NULL;
+	Q_FOREACH (const QString& tag, ui.tagWidget->getBlockStringList())
+		taglist_add_tag(&displayed_dive.tag_list, qPrintable(tag));
+	taglist_cleanup(&displayed_dive.tag_list);
+}
+
 // changing the tags on multiple dives is semantically strange - what's the right thing to do?
 // here's what I think... add the tags that were added to the displayed dive and remove the tags
 // that were removed from it
-void MainTab::saveTags()
+void MainTab::saveTags(const QVector<dive *> &selectedDives)
 {
 	struct dive *cd = current_dive;
 	struct tag_entry *added_list = NULL;
 	struct tag_entry *removed_list = NULL;
 	struct tag_entry *tl;
 
-	taglist_free(displayed_dive.tag_list);
-	displayed_dive.tag_list = NULL;
-	Q_FOREACH (const QString& tag, ui.tagWidget->getBlockStringList())
-		taglist_add_tag(&displayed_dive.tag_list, qPrintable(tag));
-	taglist_cleanup(&displayed_dive.tag_list);
+	copyTagsToDisplayedDive();
 
 	// figure out which tags were added and which tags were removed
-	added_list = taglist_added(cd->tag_list, displayed_dive.tag_list);
-	removed_list = taglist_added(displayed_dive.tag_list, cd->tag_list);
+	added_list = taglist_added(cd ? cd->tag_list : NULL, displayed_dive.tag_list);
+	removed_list = taglist_added(displayed_dive.tag_list, cd ? cd->tag_list : NULL);
+
 	// dump_taglist("added tags:", added_list);
 	// dump_taglist("removed tags:", removed_list);
 
@@ -1270,7 +1214,7 @@ void MainTab::saveTags()
 	if (added_list == NULL && removed_list == NULL)
 		return;
 
-	MODIFY_SELECTED_DIVES(
+	MODIFY_DIVES(selectedDives,
 		// create a new tag list and all the existing tags that were not
 		// removed and then all the added tags
 		struct tag_entry *new_tag_list;
@@ -1296,56 +1240,56 @@ void MainTab::saveTags()
 // buddy and divemaster are represented in the UI just like the tags, but the internal
 // representation is just a string (with commas as delimiters). So we need to do the same
 // thing we did for tags, just differently
-void MainTab::saveTaggedStrings()
+void MainTab::saveTaggedStrings(const QVector<dive *> &selectedDives)
 {
 	QStringList addedList, removedList;
 	struct dive *cd = current_dive;
 
-	diffTaggedStrings(cd->buddy, displayed_dive.buddy, addedList, removedList);
-	MODIFY_SELECTED_DIVES(
-		QStringList oldList = QString(mydive->buddy).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
-		QString newString;
-		QString comma;
-		Q_FOREACH (const QString tag, oldList) {
-			if (!removedList.contains(tag, Qt::CaseInsensitive)) {
-				newString += comma + tag;
-				comma = ", ";
+	if (diffTaggedStrings(cd->buddy, displayed_dive.buddy, addedList, removedList))
+		MODIFY_DIVES(selectedDives,
+			QStringList oldList = QString(mydive->buddy).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
+			QString newString;
+			QString comma;
+			Q_FOREACH (const QString tag, oldList) {
+				if (!removedList.contains(tag, Qt::CaseInsensitive)) {
+					newString += comma + tag;
+					comma = ", ";
+				}
 			}
-		}
-		Q_FOREACH (const QString tag, addedList) {
-			if (!oldList.contains(tag, Qt::CaseInsensitive)) {
-				newString += comma + tag;
-				comma = ", ";
+			Q_FOREACH (const QString tag, addedList) {
+				if (!oldList.contains(tag, Qt::CaseInsensitive)) {
+					newString += comma + tag;
+					comma = ", ";
+				}
 			}
-		}
-		free(mydive->buddy);
-		mydive->buddy = copy_qstring(newString);
-	);
+			free(mydive->buddy);
+			mydive->buddy = copy_qstring(newString);
+		);
 	addedList.clear();
 	removedList.clear();
-	diffTaggedStrings(cd->divemaster, displayed_dive.divemaster, addedList, removedList);
-	MODIFY_SELECTED_DIVES(
-		QStringList oldList = QString(mydive->divemaster).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
-		QString newString;
-		QString comma;
-		Q_FOREACH (const QString tag, oldList) {
-			if (!removedList.contains(tag, Qt::CaseInsensitive)) {
-				newString += comma + tag;
-				comma = ", ";
+	if (diffTaggedStrings(cd->divemaster, displayed_dive.divemaster, addedList, removedList))
+		MODIFY_DIVES(selectedDives,
+			QStringList oldList = QString(mydive->divemaster).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
+			QString newString;
+			QString comma;
+			Q_FOREACH (const QString tag, oldList) {
+				if (!removedList.contains(tag, Qt::CaseInsensitive)) {
+					newString += comma + tag;
+					comma = ", ";
+				}
 			}
-		}
-		Q_FOREACH (const QString tag, addedList) {
-			if (!oldList.contains(tag, Qt::CaseInsensitive)) {
-				newString += comma + tag;
-				comma = ", ";
+			Q_FOREACH (const QString tag, addedList) {
+				if (!oldList.contains(tag, Qt::CaseInsensitive)) {
+					newString += comma + tag;
+					comma = ", ";
+				}
 			}
-		}
-		free(mydive->divemaster);
-		mydive->divemaster = copy_qstring(newString);
-	);
+			free(mydive->divemaster);
+			mydive->divemaster = copy_qstring(newString);
+		);
 }
 
-void MainTab::diffTaggedStrings(QString currentString, QString displayedString, QStringList &addedList, QStringList &removedList)
+int MainTab::diffTaggedStrings(QString currentString, QString displayedString, QStringList &addedList, QStringList &removedList)
 {
 	QStringList displayedList, currentList;
 	currentList = currentString.split(',', QString::SkipEmptyParts);
@@ -1358,6 +1302,7 @@ void MainTab::diffTaggedStrings(QString currentString, QString displayedString,
 		if (!currentList.contains(tag, Qt::CaseInsensitive))
 			addedList << tag.trimmed();
 	}
+	return removedList.length() + addedList.length();
 }
 
 void MainTab::on_tagWidget_textChanged()
@@ -1379,7 +1324,7 @@ void MainTab::on_location_textChanged()
 	// we don't want to act on the edit until editing is finished,
 	// but we want to mark the field so it's obvious it is being edited
 	QString currentLocation;
-	struct dive_site *ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
+	struct dive_site *ds = displayed_dive.dive_site;
 	if (ds)
 		currentLocation = ds->name;
 	if (ui.location->text() != currentLocation)
@@ -1392,12 +1337,12 @@ void MainTab::on_location_diveSiteSelected()
 		return;
 
 	if (ui.location->text().isEmpty()) {
-		displayed_dive.dive_site_uuid = 0;
+		displayed_dive.dive_site = nullptr;
 		markChangedWidget(ui.location);
-		emit diveSiteChanged(0);
+		emit diveSiteChanged();
 		return;
 	} else {
-		if (ui.location->currDiveSiteUuid() != displayed_dive.dive_site_uuid) {
+		if (ui.location->currDiveSite() != displayed_dive.dive_site) {
 			markChangedWidget(ui.location);
 		} else {
 			QPalette p;
@@ -1467,7 +1412,7 @@ void MainTab::on_visibility_valueChanged(int value)
 	}
 }
 
-#undef MODIFY_SELECTED_DIVES
+#undef MODIFY_DIVES
 #undef EDIT_TEXT
 #undef EDIT_VALUE
 
@@ -1534,7 +1479,7 @@ void MainTab::showAndTriggerEditSelective(struct dive_components what)
 	if (what.visibility)
 		ui.visibility->setCurrentStars(displayed_dive.visibility);
 	if (what.divesite)
-		ui.location->setCurrentDiveSiteUuid(displayed_dive.dive_site_uuid);
+		ui.location->setCurrentDiveSite(displayed_dive.dive_site);
 	if (what.tags) {
 		ui.tagWidget->setText(get_taglist_string(displayed_dive.tag_list));
 	}
