@@ -49,17 +49,9 @@ int main(int argc, char **argv)
 
 	// this should have set up the informational preferences - let's grab
 	// the units from there
-
 	prefs.unit_system = git_prefs.unit_system;
 	prefs.units = git_prefs.units;
 
-	// populate the statistics
-	struct dive *d = get_dive(0);
-	struct dive *pd;
-	if (d) {
-		process_all_dives(d, &pd);
-	}
-
 	// now set up the export settings to create the HTML export
 	struct htmlExportSetting hes;
 	hes.themeFile = "sand.css";
