@@ -15,6 +15,8 @@
 #include <QDateTime>
 #include <QClipboard>
 #include <QFile>
+#include <QtConcurrent>
+#include <QFuture>
 
 #include <QBluetoothLocalDevice>
 
@@ -325,8 +327,9 @@ void QMLManager::updateAllGlobalLists()
 void QMLManager::mergeLocalRepo()
 {
 	char *filename = NOCLOUD_LOCALSTORAGE;
-	parse_file(filename, &dive_table);
-	process_imported_dives(false);
+	struct dive_table table = { 0 };
+	parse_file(filename, &table);
+	process_imported_dives(&table, false, false);
 }
 
 void QMLManager::copyAppLogToClipboard()
@@ -374,7 +377,7 @@ void QMLManager::finishSetup()
 	// Initialize cloud credentials.
 	QMLPrefs::instance()->setCloudUserName(qPrefCloudStorage::cloud_storage_email());
 	QMLPrefs::instance()->setCloudPassword(qPrefCloudStorage::cloud_storage_password());
-	setSyncToCloud(!git_local_only);
+	git_local_only = !prefs.cloud_auto_sync;
 	QMLPrefs::instance()->setCredentialStatus((qPrefCloudStorage::cloud_status) prefs.cloud_verification_status);
 	// if the cloud credentials are valid, we should get the GPS Webservice ID as well
 	QString url;
@@ -671,16 +674,16 @@ successful_exit:
 	// for the remote data - which then later gets merged with the remote data if necessary
 	if (noCloudToCloud) {
 		git_storage_update_progress(qPrintable(tr("Loading dives from local storage ('no cloud' mode)")));
-		dive_table.preexisting = dive_table.nr;
 		mergeLocalRepo();
 		DiveListModel::instance()->clear();
 		DiveListModel::instance()->addAllDives();
 		appendTextToLog(QStringLiteral("%1 dives loaded after importing nocloud local storage").arg(dive_table.nr));
 		noCloudToCloud = false;
+		mark_divelist_changed(true);
 		saveChangesLocal();
-		if (m_syncToCloud == false) {
+		if (git_local_only == false) {
 			appendTextToLog(QStringLiteral("taking things back offline now that storage is synced"));
-			git_local_only = m_syncToCloud;
+			git_local_only = true;
 		}
 	}
 	// if we got here just for an initial connection to the cloud, reset to offline
@@ -704,9 +707,9 @@ void QMLManager::revertToNoCloudIfNeeded()
 		// and cloud data) failed - so let's delete the cloud credentials and go
 		// back to CS_NOCLOUD mode in order to prevent us from losing the locally stored
 		// dives
-		if (m_syncToCloud == false) {
+		if (git_local_only == true) {
 			appendTextToLog(QStringLiteral("taking things back offline since sync with cloud failed"));
-			git_local_only = m_syncToCloud;
+			git_local_only = false;
 		}
 		free((void *)prefs.cloud_storage_email);
 		prefs.cloud_storage_email = NULL;
@@ -753,14 +756,11 @@ void QMLManager::refreshDiveList()
 
 static void setupDivesite(struct dive *d, struct dive_site *ds, double lat, double lon, const char *locationtext)
 {
+	location_t location = create_location(lat, lon);
 	if (ds) {
-		ds->latitude.udeg = lrint(lat * 1000000);
-		ds->longitude.udeg = lrint(lon * 1000000);
+		ds->location = location;
 	} else {
-		degrees_t latData, lonData;
-		latData.udeg = lrint(lat * 1000000);
-		lonData.udeg = lrint(lon * 1000000);
-		d->dive_site_uuid = create_dive_site_with_gps(locationtext, latData, lonData, d->when);
+		d->dive_site = create_dive_site_with_gps(locationtext, &location, d->when);
 	}
 }
 
@@ -872,16 +872,16 @@ parsed:
 bool QMLManager::checkLocation(DiveObjectHelper *myDive, struct dive *d, QString location, QString gps)
 {
 	bool diveChanged = false;
-	uint32_t uuid = 0;
 	struct dive_site *ds = get_dive_site_for_dive(d);
 	qDebug() << "checkLocation" << location << "gps" << gps << "dive had" << myDive->location() << "gps" << myDive->gas();
 	if (myDive->location() != location) {
 		diveChanged = true;
 		if (!ds)
-			uuid = get_dive_site_uuid_by_name(qPrintable(location), NULL);
-		if (!uuid && !location.isEmpty())
-			uuid = create_dive_site(qPrintable(location), d->when);
-		d->dive_site_uuid = uuid;
+			ds = get_dive_site_by_name(qPrintable(location));
+		if (!ds && !location.isEmpty()) {
+			ds = create_dive_site(qPrintable(location), d->when);
+		}
+		d->dive_site = ds;
 	}
 	// now make sure that the GPS coordinates match - if the user changed the name but not
 	// the GPS coordinates, this still does the right thing as the now new dive site will
@@ -1093,6 +1093,9 @@ void QMLManager::commitChanges(QString diveId, QString date, QString location, Q
 		d->buddy = copy_qstring(buddy);
 	}
 	if (myDive->divemaster() != diveMaster) {
+		if (diveMaster.contains(",")){
+			diveMaster = diveMaster.replace(QRegExp("\\s*,\\s*"), ", ");
+		}
 		diveChanged = true;
 		free(d->divemaster);
 		d->divemaster = copy_qstring(diveMaster);
@@ -1294,14 +1297,12 @@ void QMLManager::deleteDive(int id)
 		appendTextToLog("trying to delete non-existing dive");
 		return;
 	}
-	// clean up (or create) the storage for the deleted dive and trip (if applicable)
+	// create the storage for the deleted dive and trip (if applicable)
 	if (!deletedDive)
 		deletedDive = alloc_dive();
-	else
-		clear_dive(deletedDive);
 	copy_dive(d, deletedDive);
 	if (!deletedTrip) {
-		deletedTrip = (struct dive_trip *)calloc(1, sizeof(struct dive_trip));
+		deletedTrip = alloc_trip();
 	} else {
 		free(deletedTrip->location);
 		free(deletedTrip->notes);
@@ -1476,15 +1477,13 @@ QString QMLManager::getVersion() const
 }
 
 QString QMLManager::getGpsFromSiteName(const QString& siteName)
-{	uint32_t uuid;
+{
 	struct dive_site *ds;
 
-	uuid = get_dive_site_uuid_by_name(qPrintable(siteName), NULL);
-	if (uuid) {
-		ds = get_dive_site_by_uuid(uuid);
-		return QString(printGPSCoords(ds->latitude.udeg, ds->longitude.udeg));
-	}
-	return "";
+	ds = get_dive_site_by_name(qPrintable(siteName));
+	if (ds)
+		return QString(printGPSCoords(&ds->location));
+	return QString();
 }
 
 void QMLManager::setNotificationText(QString text)
@@ -1493,13 +1492,6 @@ void QMLManager::setNotificationText(QString text)
 	emit notificationTextChanged();
 }
 
-void QMLManager::setSyncToCloud(bool status)
-{
-	m_syncToCloud = status;
-	git_local_only = !status;
-	emit syncToCloudChanged();
-}
-
 void QMLManager::setUpdateSelectedDive(int idx)
 {
 	m_updateSelectedDive = idx;
@@ -1765,6 +1757,11 @@ int QMLManager::getConnectionIndex(const QString &deviceSubstr)
 	return connectionListModel.indexOf(deviceSubstr);
 }
 
+void QMLManager::setGitLocalOnly(const bool &value)
+{
+	git_local_only = value;
+}
+
 void QMLManager::showDownloadPage(QString deviceString)
 {
 	// we pass the indices for the three combo boxes for vendor, product, and connection
@@ -1850,6 +1847,17 @@ void QMLManager::showDownloadPage(QString deviceString)
 	emit pluggedInDeviceNameChanged();
 }
 
+void QMLManager::setFilter(const QString filterText)
+{
+	// show that we are doing something, then do something in another thread in order not to block the UI
+	QMetaObject::invokeMethod(qmlWindow, "showBusy");
+	QtConcurrent::run(QThreadPool::globalInstance(),
+			  [=]{
+				dlSortModel->setFilter(filterText);
+				QMetaObject::invokeMethod(qmlWindow, "hideBusy");
+			  });
+}
+
 #if defined(Q_OS_ANDROID)
 // implemented in core/android.cpp
 void checkPendingIntents();
