@@ -28,7 +28,6 @@ class QMLManager : public QObject {
 	Q_PROPERTY(QString startPageText MEMBER m_startPageText WRITE setStartPageText NOTIFY startPageTextChanged)
 	Q_PROPERTY(bool verboseEnabled MEMBER m_verboseEnabled WRITE setVerboseEnabled NOTIFY verboseEnabledChanged)
 	Q_PROPERTY(QString notificationText MEMBER m_notificationText WRITE setNotificationText NOTIFY notificationTextChanged)
-	Q_PROPERTY(bool syncToCloud MEMBER m_syncToCloud WRITE setSyncToCloud NOTIFY syncToCloudChanged)
 	Q_PROPERTY(int updateSelectedDive MEMBER m_updateSelectedDive WRITE setUpdateSelectedDive NOTIFY updateSelectedDiveChanged)
 	Q_PROPERTY(int selectedDiveTimestamp MEMBER m_selectedDiveTimestamp WRITE setSelectedDiveTimestamp NOTIFY selectedDiveTimestampChanged)
 	Q_PROPERTY(QStringList suitList READ suitList NOTIFY suitListChanged)
@@ -88,6 +87,8 @@ public:
 	Q_INVOKABLE int getDetectedVendorIndex();
 	Q_INVOKABLE int getDetectedProductIndex(const QString &currentVendorText);
 	Q_INVOKABLE int getConnectionIndex(const QString &deviceSubstr);
+	Q_INVOKABLE void setGitLocalOnly(const bool &value);
+	Q_INVOKABLE void setFilter(const QString filterText);
 
 	static QMLManager *instance();
 	Q_INVOKABLE void registerError(QString error);
@@ -115,9 +116,6 @@ public:
 	QString notificationText() const;
 	void setNotificationText(QString text);
 
-	bool syncToCloud() const;
-	void setSyncToCloud(bool status);
-
 	int updateSelectedDive() const;
 	void setUpdateSelectedDive(int idx);
 
@@ -139,6 +137,7 @@ public:
 	QStringList cylinderInit() const;
 	Q_INVOKABLE void setStatusbarColor(QColor color);
 	void btHostModeChange(QBluetoothLocalDevice::HostMode state);
+	QObject *qmlWindow;
 
 #if defined(Q_OS_ANDROID) || defined(Q_OS_IOS)
 	void writeToAppLogFile(QString logText);
@@ -210,7 +209,6 @@ private:
 	struct dive *deletedDive;
 	struct dive_trip *deletedTrip;
 	QString m_notificationText;
-	bool m_syncToCloud;
 	int m_updateSelectedDive;
 	int m_selectedDiveTimestamp;
 	qreal m_lastDevicePixelRatio;
@@ -241,7 +239,6 @@ signals:
 	void loadFromCloudChanged();
 	void startPageTextChanged();
 	void notificationTextChanged();
-	void syncToCloudChanged();
 	void updateSelectedDiveChanged();
 	void selectedDiveTimestampChanged();
 	void sendScreenChanged(QScreen *screen);
