@@ -17,7 +17,7 @@
 # JOBS := 1
 #
 # # This variable controls the targets that will build.
-# MXE_TARGETS :=  i686-w64-mingw32.shared
+# MXE_TARGETS :=  i686-w64-mingw32.shared.posix.dw2
 #
 # # Uncomment the next line if you want to do debug builds later
 # # qtbase_CONFIGURE_OPTS=-debug-and-release
@@ -27,7 +27,11 @@
 #
 # now you can start the build
 #
-# make libxml2 libxslt libusb1 libzip libssh2 curl qt5 nsis
+# make libxml2 libxslt libusb1 libzip libssh2 libftdi1 curl qt5 nsis
+#
+#     (if you intend to build Subsurface without user space FTDI support
+#      you can drop libftdi1 from that list and start this script with
+#      -noftdi )
 #
 # After quite a while (depending on your machine anywhere from 15-20
 # minutes to several hours) you should have a working MXE install in
@@ -42,7 +46,6 @@
 # ~/src/mxe                    <- MXE git with Qt5, automake (see above)
 #      /grantlee               <- Grantlee 5.0.0 sources from git
 #      /subsurface             <- current subsurface git
-#      /libgit2                <- libgit2 0.23.1 or similar
 #      /googlemaps             <- Google Maps plugin for QtLocation from git
 #
 # ~/src/win32                  <- build directory
@@ -83,6 +86,13 @@ exec 1> >(tee ./winbuild.log) 2>&1
 # for debugging
 #trap "set +x; sleep 1; set -x" DEBUG
 
+if [[ "$1" == "-noftdi" ]] ; then
+	shift
+	FTDI="OFF"
+else
+	FTDI="ON"
+fi
+
 # this is run on a rather powerful machine - if you want less
 # build parallelism, please change this variable
 JOBS="-j4"
@@ -91,6 +101,7 @@ EXECDIR=`pwd`
 BASEDIR=$(cd "$EXECDIR/.."; pwd)
 BUILDDIR=$(cd "$EXECDIR"; pwd)
 MXEDIR=${MXEDIR:-mxe}
+MXEBUILDTYPE=${MXEBUILDTYPE:-i686-w64-mingw32.shared.posix.dw2}
 
 echo $BUILDDIR
 
@@ -102,7 +113,7 @@ fi
 
 echo "Building in $BUILDDIR ..."
 
-export PATH="$BASEDIR"/"$MXEDIR"/usr/bin:$PATH:"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/
+export PATH="$BASEDIR"/"$MXEDIR"/usr/bin:$PATH:"$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/bin/
 export CXXFLAGS=-std=c++11
 
 if [[ "$1" == "debug" ]] ; then
@@ -133,7 +144,7 @@ if [[ ! -d grantlee || -f build.grantlee ]] ; then
 	rm -f build.grantlee
 	mkdir -p grantlee
 	cd grantlee
-	i686-w64-mingw32.shared-cmake \
+	"$MXEBUILDTYPE"-cmake \
 		-DCMAKE_BUILD_TYPE=$RELEASE \
 		-DBUILD_TESTS=OFF \
 		"$BASEDIR"/grantlee
@@ -142,22 +153,6 @@ if [[ ! -d grantlee || -f build.grantlee ]] ; then
 	make install
 fi
 
-# libgit2:
-
-cd "$BUILDDIR"
-if [[ ! -d libgit2 || -f build.libgit2 ]] ; then
-	rm -f build.libgit2
-	mkdir -p libgit2
-	cd libgit2
-	i686-w64-mingw32.shared-cmake \
-		-DBUILD_CLAR=OFF -DTHREADSAFE=ON \
-		-DCMAKE_BUILD_TYPE=$RELEASE \
-		-DDLLTOOL="$BASEDIR"/"$MXEDIR"/usr/bin/i686-w64-mingw32.shared-dlltool \
-		"$BASEDIR"/libgit2
-	make $JOBS
-	make install
-fi
-
 # libdivecomputer
 # ensure the git submodule is present and the autotools are set up
 
@@ -181,10 +176,10 @@ if [ ! "$CURRENT_SHA" = "$PREVIOUS_SHA" ] || [ ! -d libdivecomputer ] || [ -f bu
 	cd libdivecomputer
 
 	"$BASEDIR"/subsurface/libdivecomputer/configure \
-		CC=i686-w64-mingw32.shared-gcc \
-		--host=i686-w64-mingw32.shared \
+		CC="$MXEBUILDTYPE"-gcc \
+		--host="$MXEBUILDTYPE" \
 		--enable-shared \
-		--prefix="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared
+		--prefix="$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"
 	make $JOBS
 	make install
 else
@@ -205,7 +200,7 @@ if [[ ! -d googlemaps || -f build.googlemaps ]] ; then
 	cd "$BUILDDIR"
 	mkdir -p googlemaps
 	cd googlemaps
-	"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/qmake PREFIX=$"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared "$BASEDIR"/googlemaps/googlemaps.pro
+	"$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/bin/qmake PREFIX=$"$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE" "$BASEDIR"/googlemaps/googlemaps.pro
 	make $JOBS $RELEASE_GM
 	make "$RELEASE_GM"-install
 fi
@@ -221,11 +216,11 @@ echo "Starting Subsurface Build"
 rm -rf subsurface
 
 # first copy the Qt plugins in place
-QT_PLUGIN_DIRECTORIES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/iconengines \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/imageformats \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/platforms \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/geoservices \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/printsupport"
+QT_PLUGIN_DIRECTORIES="$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/plugins/iconengines \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/plugins/imageformats \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/plugins/platforms \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/plugins/geoservices \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/plugins/printsupport"
 
 STAGING_DIR=$BUILDDIR/subsurface/staging
 STAGING_TESTS_DIR=$BUILDDIR/subsurface/staging_tests
@@ -253,9 +248,9 @@ do
 done
 
 # next we need the QML modules
-QT_QML_MODULES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtQuick.2 \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtLocation \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtPositioning"
+QT_QML_MODULES="$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/qml/QtQuick.2 \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/qml/QtLocation \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/qml/QtPositioning"
 
 mkdir -p $STAGING_DIR/qml
 
@@ -266,8 +261,8 @@ done
 
 # for some reason we aren't installing Qt5Xml.dll and Qt5Location.dll
 # I need to figure out why and fix that, but for now just manually copy that as well
-EXTRA_MANUAL_DEPENDENCIES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/Qt5Xml$DLL_SUFFIX.dll \
-$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/Qt5Location$DLL_SUFFIX.dll"
+EXTRA_MANUAL_DEPENDENCIES="$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/bin/Qt5Xml$DLL_SUFFIX.dll \
+$BASEDIR/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/bin/Qt5Location$DLL_SUFFIX.dll"
 
 for f in $EXTRA_MANUAL_DEPENDENCIES
 do
@@ -277,14 +272,17 @@ done
 
 cd "$BUILDDIR"/subsurface
 
-i686-w64-mingw32.shared-cmake \
-	-DCMAKE_PREFIX_PATH="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5 \
+"$MXEBUILDTYPE"-cmake \
+	-DCMAKE_PREFIX_PATH="$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5 \
 	-DCMAKE_BUILD_TYPE=$RELEASE_MAIN \
-	-DQT_TRANSLATION_DIR="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/translations \
-	-DMAKENSIS=i686-w64-mingw32.shared-makensis \
-	-DLIBDIVECOMPUTER_INCLUDE_DIR="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/include \
-	-DLIBDIVECOMPUTER_LIBRARIES="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/lib/libdivecomputer.dll.a \
+	-DQT_TRANSLATION_DIR="$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/qt5/translations \
+	-DMAKENSIS="$MXEBUILDTYPE"-makensis \
+	-DLIBDIVECOMPUTER_INCLUDE_DIR="$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/include \
+	-DLIBDIVECOMPUTER_LIBRARIES="$BASEDIR"/"$MXEDIR"/usr/"$MXEBUILDTYPE"/lib/libdivecomputer.dll.a \
 	-DMAKE_TESTS=OFF \
+	-DBTSUPPORT=ON -DBLESUPPORT=ON \
+	-DFTDISUPPORT=$FTDI \
+	-DLIBGIT2_FROM_PKGCONFIG=ON \
 	"$BASEDIR"/subsurface
 
 make $JOBS "$@"
