@@ -90,7 +90,7 @@ if [ $PLATFORM = Darwin ] ; then
 	BASESDK=$(ls $SDKROOT | grep "MacOSX10\.1.\.sdk" | head -1 | sed -e "s/MacOSX//;s/\.sdk//")
 	OLDER_MAC="-mmacosx-version-min=10.11 -isysroot${SDKROOT}/MacOSX${BASESDK}.sdk"
 	OLDER_MAC_CMAKE="-DCMAKE_OSX_DEPLOYMENT_TARGET=${BASESDK} -DCMAKE_OSX_SYSROOT=${SDKROOT}/MacOSX${BASESDK}.sdk/"
-	if [ ! -d /usr/include ] ; then
+	if [[ ! -d /usr/include && ! -d "${SDKROOT}/MacOSX.sdk/usr/include" ]] ; then
 		echo "Error: Xcode Command Line Tools are not installed"
 		echo ""
 		echo "Please run:"
@@ -346,14 +346,22 @@ if [ ! -f ../configure ] ; then
 fi
 CFLAGS="$OLDER_MAC -I$INSTALL_ROOT/include $LIBDC_CFLAGS" ../configure --prefix=$INSTALL_ROOT --disable-examples
 if [ $PLATFORM = Darwin ] ; then
-	# it seems that on my Mac some of the configure tests for libdivecomputer
-	# pass even though the feature tested for is actually missing
-	# let's hack around that
-	sed -i .bak 's/^#define HAVE_CLOCK_GETTIME 1/\/* #undef HAVE_CLOCK_GETTIME *\//' config.h
+	# remove some copmpiler options that aren't supported on Mac
+	# otherwise the log gets very noisy
 	for i in $(find . -name Makefile)
 	do
 		sed -i .bak 's/-Wrestrict//;s/-Wno-unused-but-set-variable//' $i
 	done
+	# it seems that on my Mac some of the configure tests for libdivecomputer
+	# pass even though the feature tested for is actually missing
+	# let's hack around that
+	# touch config.status, recreate config.h and then disable HAVE_CLOCK_GETTIME
+	# this seems to work so that the Makefile doesn't re-run the
+	# configure process and overwrite all the changes we just made
+	touch config.status
+	make config.h
+	grep CLOCK_GETTIME config.h
+	sed -i .bak 's/^#define HAVE_CLOCK_GETTIME 1/#undef HAVE_CLOCK_GETTIME /' config.h
 fi
 make -j4
 make install
@@ -398,19 +406,8 @@ if [ "$BUILDGRANTLEE" = "1" ] ; then
 	PRINTING="-DNO_PRINTING=OFF"
 
 	cd $SRC
-
-	if [ ! -d grantlee ] ; then
-		if [[ $1 = local ]] ; then
-			git clone $SRC/../grantlee grantlee
-		else
-			git clone https://github.com/steveire/grantlee.git
-		fi
-	fi
-	cd grantlee
-	if ! git checkout v5.0.0 ; then
-		echo "can't check out v5.0.0 of grantlee -- giving up"
-		exit 1
-	fi
+	./subsurface/scripts/get-dep-lib.sh single . grantlee
+	pushd grantlee
 	mkdir -p build
 	cd build
 	cmake $OLDER_MAC_CMAKE -DCMAKE_BUILD_TYPE=$DEBUGRELEASE \
@@ -419,6 +416,7 @@ if [ "$BUILDGRANTLEE" = "1" ] ; then
 		$SRC/grantlee
 	make -j4
 	make install
+	popd
 else
 	PRINTING="-DNO_PRINTING=ON"
 fi
@@ -427,29 +425,22 @@ if [ "$SKIP_GOOGLEMAPS" != "1" ] ; then
 	# build the googlemaps map plugin
 
 	cd $SRC
-	if [ ! -d googlemaps ] ; then
-		if [[ $1 = local ]] ; then
-			git clone $SRC/../googlemaps googlemaps
-		else
-			git clone https://github.com/Subsurface-divelog/googlemaps.git
-		fi
-	fi
-	cd googlemaps
-	git checkout master
-	git pull --rebase
-
+	./subsurface/scripts/get-dep-lib.sh single . googlemaps
+	pushd googlemaps
 	mkdir -p build
 	mkdir -p J10build
 	cd build
-	$QMAKE -query
 	$QMAKE "INCLUDEPATH=$INSTALL_ROOT/include" ../googlemaps.pro
 	# on Travis the compiler doesn't support c++1z, yet qmake adds that flag;
 	# since things compile fine with c++11, let's just hack that away
 	# similarly, don't use -Wdata-time
-	mv Makefile Makefile.bak
-	cat Makefile.bak | sed -e 's/std=c++1z/std=c++11/g ; s/-Wdate-time//' > Makefile
+	if [ "$TRAVIS" = "true" ] ; then
+		mv Makefile Makefile.bak
+		cat Makefile.bak | sed -e 's/std=c++1z/std=c++11/g ; s/-Wdate-time//' > Makefile
+	fi
 	make -j4
 	make install
+	popd
 fi
 
 # finally, build Subsurface
