@@ -12,10 +12,10 @@ find . -name subsurface\*.exe*
 # set up the release message to use
 source ${TRAVIS_BUILD_DIR}/scripts/release-message.sh
 
-# get and run the upload script
+# get the upload script
 wget -c https://raw.githubusercontent.com/dirkhh/uploadtool/master/upload.sh
-bash ./upload.sh subsurface*.exe*
 
-# upload smtk2ssrf
+# don't upload the Subsurface build (we get that from the container based
+# Windows build, just upload smtk2ssrf
 cd ${TRAVIS_BUILD_DIR}/../win32/smtk-import
 bash ../subsurface/upload.sh smtk2ssrf*.exe*
