@@ -315,7 +315,7 @@ static void smtk_wreck_site(MdbHandle *mdb, char *site_idx, struct dive_site *ds
  * Location format:
  * | Idx | Text | Province | Country | Depth |
  */
-static void smtk_build_location(MdbHandle *mdb, char *idx, timestamp_t when, uint32_t *location)
+static void smtk_build_location(MdbHandle *mdb, char *idx, timestamp_t when, struct dive_site **location)
 {
 	MdbTableDef *table;
 	MdbColumn *col[MDB_MAX_COLS];
@@ -323,7 +323,7 @@ static void smtk_build_location(MdbHandle *mdb, char *idx, timestamp_t when, uin
 	int i;
 	uint32_t d;
 	struct dive_site *ds;
-	degrees_t lat, lon;
+	location_t loc;
 	char *str = NULL, *loc_idx = NULL, *site = NULL, *notes = NULL;
 	const char *site_fields[] = {QT_TRANSLATE_NOOP("gettextFromC", "Altitude"), QT_TRANSLATE_NOOP("gettextFromC", "Depth"),
 				     QT_TRANSLATE_NOOP("gettextFromC", "Notes")};
@@ -337,8 +337,7 @@ static void smtk_build_location(MdbHandle *mdb, char *idx, timestamp_t when, uin
 		mdb_fetch_row(table);
 	loc_idx = copy_string(col[2]->bind_ptr);
 	site = copy_string(col[1]->bind_ptr);
-	lat.udeg = lrint(strtod(col[6]->bind_ptr, NULL) * 1000000);
-	lon.udeg = lrint(strtod(col[7]->bind_ptr, NULL) * 1000000);
+	loc = create_location(strtod(col[6]->bind_ptr, NULL), strtod(col[7]->bind_ptr, NULL));
 
 	for (i = 8; i < 11; i++) {
 		switch (i) {
@@ -374,17 +373,17 @@ static void smtk_build_location(MdbHandle *mdb, char *idx, timestamp_t when, uin
 		str = smtk_concat_str(str, ", ", "%s", col[1]->bind_ptr); // Locality
 	str =  smtk_concat_str(str, ", ", "%s", site);
 
-	*location = get_dive_site_uuid_by_name(str, NULL);
-	if (*location == 0) {
-		if (lat.udeg == 0 && lon.udeg == 0)
-			*location = create_dive_site(str, when);
+	ds = get_dive_site_by_name(str);
+	if (!ds) {
+		if (!has_location(&loc))
+			ds = create_dive_site(str, when);
 		else
-			*location = create_dive_site_with_gps(str, lat, lon, when);
+			ds = create_dive_site_with_gps(str, &loc, when);
 	}
+	*location = ds;
 	smtk_free(bound_values, table->num_cols);
 
 	/* Insert site notes */
-	ds = get_dive_site_by_uuid(*location);
 	ds->notes = copy_string(notes);
 	free(notes);
 
@@ -1064,7 +1063,7 @@ void smartrak_import(const char *file, struct dive_table *divetable)
 		smtkdive->visibility = strtod(col[coln(VISIBILITY)]->bind_ptr, NULL) > 25 ? 5 : lrint(strtod(col[13]->bind_ptr, NULL) / 5);
 		smtkdive->weightsystem[0].weight.grams = lrint(strtod(col[coln(WEIGHT)]->bind_ptr, NULL) * 1000);
 		smtkdive->suit = copy_string(suit_list[atoi(col[coln(SUITIDX)]->bind_ptr) - 1]);
-		smtk_build_location(mdb_clon, col[coln(SITEIDX)]->bind_ptr, smtkdive->when, &smtkdive->dive_site_uuid);
+		smtk_build_location(mdb_clon, col[coln(SITEIDX)]->bind_ptr, smtkdive->when, &smtkdive->dive_site);
 		smtkdive->buddy = smtk_locate_buddy(mdb_clon, col[0]->bind_ptr, buddy_list);
 		smtk_parse_relations(mdb_clon, smtkdive, col[0]->bind_ptr, "Type", "TypeRelation", type_list, true);
 		smtk_parse_relations(mdb_clon, smtkdive, col[0]->bind_ptr, "Activity", "ActivityRelation", activity_list, false);
